@@ -1,7 +1,7 @@
 /*
  * FilePath.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -31,6 +31,7 @@
 #include <boost/algorithm/string/predicate.hpp>
 
 #include <boost/bind.hpp>
+#include <boost/make_shared.hpp>
 #include <boost/iostreams/stream.hpp>
 #include <boost/iostreams/device/file_descriptor.hpp>
 
@@ -52,10 +53,7 @@ namespace {
 // - On Windows, we use Filesystem v3 with wide character paths. This is
 //   because narrow character paths on Windows can't cover the entire
 //   Unicode space since there is no ANSI code page for UTF-8.
-// - On non-Windows, if Filesystem v3 is available, we use it.
-// - Otherwise, we use Filesystem v2. This is necessary for older versions
-//   of Boost that come preinstalled on some long-term-stable Linux distro
-//   versions that we still want to support.
+// - On non-Windows, we use Filesystem v3.
 #ifdef _WIN32
 
 #define BOOST_FS_STRING(path) toString((path).generic_wstring())
@@ -75,14 +73,7 @@ typedef boost::filesystem::directory_iterator dir_iterator;
 typedef boost::filesystem::recursive_directory_iterator recursive_dir_iterator;
 
 #else
-
-#define BOOST_FS_STRING(str) (str)
-#define BOOST_FS_PATH2STR(str) ((str).string())
-#define BOOST_FS_PATH2STRNATIVE(str) ((str).string())
-#define BOOST_FS_COMPLETE(p, base) boost::filesystem::complete(p, base)
-typedef boost::filesystem::basic_directory_iterator<path_t> dir_iterator;
-typedef boost::filesystem::basic_recursive_directory_iterator<path_t> recursive_dir_iterator;
-
+#error FilePath requires Filesystem v3
 #endif
 
 
@@ -143,6 +134,13 @@ bool copySingleItem(const FilePath& from, const FilePath& to,
    return true;
 }
 
+bool addItemSize(const FilePath& item, boost::shared_ptr<int> pTotal)
+{
+   if (!item.isDirectory())
+      *pTotal = *pTotal + item.size();
+   return true;
+}
+
 }
 
 struct FilePath::Impl
@@ -163,10 +161,8 @@ FilePath FilePath::safeCurrentPath(const FilePath& revertToPath)
    {
 #ifdef _WIN32
       return FilePath(boost::filesystem::current_path().wstring()) ;
-#elif defined(BOOST_FILESYSTEM_VERSION) && BOOST_FILESYSTEM_VERSION != 2
-      return FilePath(boost::filesystem::current_path().string()) ;
 #else
-      return FilePath(boost::filesystem::current_path<path_t>().string()) ;
+      return FilePath(boost::filesystem::current_path().string()) ;
 #endif
    }
    catch(const boost::filesystem::filesystem_error& e)
@@ -344,6 +340,22 @@ bool FilePath::isSymlink() const
    }
 }
 
+FilePath FilePath::resolveSymlink() const
+{
+   try
+   {
+      if (!isSymlink())
+         return *this;
+
+      return FilePath(BOOST_FS_PATH2STR(boost::filesystem::read_symlink(pImpl_->path)));
+   }
+   catch(const boost::filesystem::filesystem_error& e)
+   {
+      logError(pImpl_->path, e, ERROR_LOCATION);
+      return *this;
+   }
+}
+
 uintmax_t FilePath::size() const
 {
    try
@@ -647,6 +659,14 @@ std::string FilePath::absolutePathNative() const
       return BOOST_FS_PATH2STRNATIVE(pImpl_->path) ;
 }
 
+std::string FilePath::canonicalPath() const
+{
+   if (empty())
+      return std::string();
+   else
+      return BOOST_FS_PATH2STR(boost::filesystem::canonical(pImpl_->path));
+}
+
 #if _WIN32
 std::wstring FilePath::absolutePathW() const
 {
@@ -839,6 +859,19 @@ Error FilePath::copyDirectoryRecursive(const FilePath& target) const
    return childrenRecursive(boost::bind(copySingleItem, *this, target, _2));
 }
 
+uintmax_t FilePath::sizeRecursive() const
+{
+   // no work to do if we're not a directory
+   if (!isDirectory())
+      return size();
+
+   boost::shared_ptr<int> pTotal = boost::make_shared<int>(0);
+   Error error = childrenRecursive(boost::bind(addItemSize, _2, pTotal)); 
+   if (error)
+      LOG_ERROR(error);
+   return *pTotal;
+}
+
 Error FilePath::ensureFile() const
 {
    // nothing to do if the file already exists
