@@ -1,7 +1,7 @@
 /*
  * REnvironmentPosix.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -28,6 +28,7 @@
 #include <core/Error.hpp>
 #include <core/FilePath.hpp>
 #include <core/ConfigUtils.hpp>
+#include <core/RegexUtils.hpp>
 #include <core/system/System.hpp>
 #include <core/system/Process.hpp>
 #include <core/system/Environment.hpp>
@@ -69,19 +70,40 @@ FilePath scanForRScript(const std::vector<std::string>& rScriptPaths,
    return FilePath();
 }
 
+// extra paths from R (for rJava)
+std::string extraLibraryPaths(const FilePath& ldPathsScript,
+                              const std::string& rHome)
+{
+   // no-op if no script is passed
+   if (ldPathsScript.empty())
+      return std::string();
+
+   // verify that script exists
+   if (!ldPathsScript.exists())
+   {
+      LOG_WARNING_MESSAGE("r-ldpath script not found at " +
+                          ldPathsScript.absolutePath());
+      return std::string();
+   }
+
+   // run script to capture paths
+   std::string command = ldPathsScript.absolutePath() + " " + rHome;
+   system::ProcessResult result;
+   Error error = runCommand(command, core::system::ProcessOptions(), &result);
+   if (error)
+      LOG_ERROR(error);
+   std::string libraryPaths = result.stdOut;
+   boost::algorithm::trim(libraryPaths);
+   
+   return libraryPaths;
+}
+
 // MacOS X Specific
 #ifdef __APPLE__
 
 #define kLibRFileName            "libR.dylib"
 #define kLibraryPathEnvVariable  "DYLD_FALLBACK_LIBRARY_PATH"
 
-// no extra paths on the mac
-std::string extraLibraryPaths(const FilePath& ldPathsScript,
-                              const std::string& rHome)
-{
-   return std::string();
-}
-
 FilePath systemDefaultRScript(std::string* pErrMsg)
 {
    // define potential paths
@@ -173,33 +195,6 @@ bool detectRLocationsUsingFramework(FilePath* pHomePath,
 #define kLibRFileName            "libR.so"
 #define kLibraryPathEnvVariable  "LD_LIBRARY_PATH"
 
-// extra paths from R (for rjava) on linux
-std::string extraLibraryPaths(const FilePath& ldPathsScript,
-                              const std::string& rHome)
-{
-   // no-op if no script is passed
-   if (ldPathsScript.empty())
-      return std::string();
-
-   // verify that script exists
-   if (!ldPathsScript.exists())
-   {
-      LOG_WARNING_MESSAGE("r-ldpaths script not found at " +
-                          ldPathsScript.absolutePath());
-      return std::string();
-   }
-
-   // run script to capture paths
-   std::string command = ldPathsScript.absolutePath() + " " + rHome;
-   system::ProcessResult result;
-   Error error = runCommand(command, core::system::ProcessOptions(), &result);
-   if (error)
-      LOG_ERROR(error);
-   std::string libraryPaths = result.stdOut;
-   boost::algorithm::trim(libraryPaths);
-   return libraryPaths;
-}
-
 FilePath systemDefaultRScript(std::string* pErrMsg)
 {
    // ask system which R to use
@@ -661,6 +656,7 @@ bool detectREnvironment(const FilePath& whichRScript,
                                           rLibPath,
                                           ldPathsScript,
                                           ldLibraryPath);
+   
    pVars->push_back(std::make_pair(kLibraryPathEnvVariable, libraryPath));
 
    // set R_ARCH on the mac if we are running against CRAN R
@@ -676,7 +672,8 @@ bool detectREnvironment(const FilePath& whichRScript,
    }
 #endif
 
-   Error error = rVersion(rHomePath, FilePath(*pRScriptPath), pVersion);
+   Error error = rVersion(rHomePath, FilePath(*pRScriptPath), ldLibraryPath, 
+         pVersion);
    if (error)
       LOG_ERROR(error);
 
@@ -732,18 +729,32 @@ std::string rLibraryPath(const FilePath& rHomePath,
                                               rHomePath.absolutePath());
    if (!extraPaths.empty())
       libraryPath.append(":" + extraPaths);
+   
    return libraryPath;
 }
 
 Error rVersion(const FilePath& rHomePath,
                const FilePath& rScriptPath,
+               const std::string& ldLibraryPath,
                std::string* pVersion)
 {
-   // determine the R version
+   // set R_HOME as provided
    core::system::ProcessOptions options;
    core::system::Options env;
    core::system::environment(&env);
    core::system::setenv(&env, "R_HOME", rHomePath.absolutePath());
+
+   // if additional LD_LIBRARY_PATH paths were supplied, provide those as well
+   if (!ldLibraryPath.empty())
+   {
+      std::string libPath = core::system::getenv(env, kLibraryPathEnvVariable);
+      if (!libPath.empty())
+         libPath += ":";
+      libPath += ldLibraryPath;
+      core::system::setenv(&env, kLibraryPathEnvVariable, libPath);
+   }
+
+   // determine the R version
    options.environment = env;
    core::system::ProcessResult result;
    Error error = core::system::runCommand(
@@ -761,7 +772,7 @@ Error rVersion(const FilePath& rHomePath,
       std::string versionInfo = boost::algorithm::trim_copy(result.stdOut);
       boost::regex re("^([\\d\\.]+)$");
       boost::smatch match;
-      if (boost::regex_search(versionInfo, match, re))
+      if (regex_utils::search(versionInfo, match, re))
       {
          *pVersion = match[1];
          return Success();
@@ -771,7 +782,12 @@ Error rVersion(const FilePath& rHomePath,
          Error error = systemError(boost::system::errc::protocol_error,
                                    "Unable to parse version from R",
                                    ERROR_LOCATION);
+
+         // log the actual output returned by R
          error.addProperty("version-info", versionInfo);
+
+         // log any errors emitted by R
+         error.addProperty("r-error", result.stdErr);
          return error;
       }
    }
