@@ -1,7 +1,7 @@
 /*
  * Win32System.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -26,6 +26,7 @@
 
 #include <windows.h>
 #include <shlobj.h>
+#include <tlhelp32.h>
 
 #include <boost/foreach.hpp>
 #include <boost/bind.hpp>
@@ -222,7 +223,12 @@ void log(LogLevel logLevel, const std::string& message)
 bool isWin64()
 {
    return !getenv("PROCESSOR_ARCHITEW6432").empty()
-         || getenv("PROCESSOR_ARCHITECTURE") == "AMD64";
+      || getenv("PROCESSOR_ARCHITECTURE") == "AMD64";
+}
+
+bool isCurrentProcessWin64()
+{
+   return getenv("PROCESSOR_ARCHITECTURE") == "AMD64";
 }
 
 bool isVistaOrLater()
@@ -242,6 +248,25 @@ bool isVistaOrLater()
    }
 }
 
+bool isWin7OrLater()
+{
+   OSVERSIONINFOA osVersion;
+   ZeroMemory(&osVersion, sizeof(OSVERSIONINFOA));
+   osVersion.dwOSVersionInfoSize = sizeof(OSVERSIONINFOA);
+
+   if (::GetVersionExA(&osVersion))
+   {
+      // 6.0 Vista, 6.1 Win7, 6.2 Win8, 6.3 Win8.1, >6 is Win10+
+      return osVersion.dwMajorVersion > 6 ||
+             osVersion.dwMajorVersion == 6 && osVersion.dwMinorVersion > 0;
+   }
+   else
+   {
+      LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+      return false;
+   }
+}
+
 std::string username()
 {
    return system::getenv("USERNAME");
@@ -498,6 +523,11 @@ Error realPath(const FilePath& filePath, FilePath* pRealPath)
    return Success();
 }
 
+Error realPath(const std::string& path, FilePath* pRealPath)
+{
+   return realPath(FilePath(path), pRealPath);
+}
+
 bool isHiddenFile(const FilePath& filePath)
 {
    return isHiddenFile(filePath.absolutePath());
@@ -801,6 +831,41 @@ void ensureLongPath(FilePath* pFilePath)
       *pFilePath = FilePath(string_utils::systemToUtf8(buffer));
    }
 }
+Error expandEnvironmentVariables(std::string value, std::string* pResult)
+{
+   if (value.empty())
+   {
+      *pResult = value;
+      return Success();
+   }
+
+   DWORD sizeRequired = ::ExpandEnvironmentStrings(value.c_str(), NULL, 0);
+   if (!sizeRequired)
+      return systemError(::GetLastError(), ERROR_LOCATION);
+
+   std::vector<char> buffer;
+   buffer.reserve(sizeRequired);
+   int result = ::ExpandEnvironmentStrings(value.c_str(),
+                                           &buffer[0],
+                                           buffer.capacity());
+
+   if (!result)
+      return systemError(GetLastError(), ERROR_LOCATION);
+   else if (result > buffer.capacity())
+      return systemError(ERROR_MORE_DATA, ERROR_LOCATION); // not expected
+
+   *pResult = std::string(&buffer[0]);
+   return Success();
+}
+
+FilePath expandComSpec()
+{
+   std::string result;
+   Error err = expandEnvironmentVariables("%COMSPEC%", &result);
+   if (err)
+      return FilePath();
+   return FilePath(result);
+}
 
 Error terminateProcess(PidType pid)
 {
@@ -812,6 +877,54 @@ Error terminateProcess(PidType pid)
    return Success();
 }
 
+std::vector<SubprocInfo> getSubprocesses(PidType pid)
+{
+   std::vector<SubprocInfo> subprocs;
+
+   HANDLE hSnapShot;
+   CloseHandleOnExitScope closeSnapShot(&hSnapShot, ERROR_LOCATION);
+
+   hSnapShot = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
+   if (hSnapShot == INVALID_HANDLE_VALUE)
+   {
+      // err on the side of assuming child processes, so we don't kill
+      // a job unintentionally
+      LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+      return subprocs;
+   }
+
+   PROCESSENTRY32 pe32;
+   pe32.dwSize = sizeof(pe32);
+   if (!Process32First(hSnapShot, &pe32))
+   {
+      LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+      return subprocs;
+   }
+
+   do
+   {
+      if (pe32.th32ParentProcessID == pid)
+      {
+         // Found a child process
+         SubprocInfo info;
+         info.pid = pe32.th32ProcessID;
+         info.exe = pe32.szExeFile;
+
+         subprocs.push_back(info);
+      }
+   } while (Process32Next(hSnapShot, &pe32));
+
+   return subprocs;
+}
+
+FilePath currentWorkingDir(PidType pid)
+{
+   // NYI for Win32; commonly accepted technique for this is to use
+   // CreateRemoteThread to inject code to run GetCurrentDirectory in the
+   // context of the target program. That is ugly and we aren't
+   // likely to ever do it.
+   return FilePath();
+}
 
 Error closeHandle(HANDLE* pHandle, const ErrorLocation& location)
 {
@@ -835,7 +948,10 @@ CloseHandleOnExitScope::~CloseHandleOnExitScope()
 {
    try
    {
-      if (!pHandle_ || *pHandle_ == INVALID_HANDLE_VALUE)
+      // A "null" handle can contain INVALID_HANDLE or NULL, depending
+      // on the context. This is a painful inconsistency in Windows, see:
+      // https://blogs.msdn.microsoft.com/oldnewthing/20040302-00/?p=40443
+      if (!pHandle_ || *pHandle_ == INVALID_HANDLE_VALUE || *pHandle_ == NULL)
          return;
 
       Error error = closeHandle(pHandle_, location_);
@@ -847,6 +963,160 @@ CloseHandleOnExitScope::~CloseHandleOnExitScope()
    }
 }
 
+struct ProcessInfo
+{
+   DWORD processId;
+   DWORD parentProcessId;
+};
+
+// simple cass to encapsulate parent-child
+// relationship of processes
+struct ProcessTreeNode
+{
+   boost::shared_ptr<ProcessInfo> data;
+   std::vector<boost::shared_ptr<ProcessTreeNode> > children;
+};
+
+// process tree, indexed by pid
+typedef std::map<DWORD, boost::shared_ptr<ProcessTreeNode> > ProcessTreeT;
+
+void createProcessTree(const std::vector<ProcessInfo>& processes,
+                       ProcessTreeT *pOutTree)
+{
+   // first pass, create the nodes in the tree
+   BOOST_FOREACH(const ProcessInfo& process, processes)
+   {
+      ProcessTreeT::iterator iter = pOutTree->find(process.processId);
+      if (iter == pOutTree->end())
+      {
+         // process not found, so create a new entry for it
+         boost::shared_ptr<ProcessTreeNode> nodePtr = boost::shared_ptr<ProcessTreeNode>(
+                                                         new ProcessTreeNode());
+
+         nodePtr->data = boost::shared_ptr<ProcessInfo>(new ProcessInfo(process));
+
+         (*pOutTree)[process.processId] = nodePtr;
+      }
+   }
+
+   // second pass, link the nodes together
+   BOOST_FOREACH(ProcessTreeT::value_type& element, *pOutTree)
+   {
+      DWORD parent = element.second->data->parentProcessId;
+      ProcessTreeT::iterator iter = pOutTree->find(parent);
+
+      // if we cannot find the parent in the tree, move on
+      if (iter == pOutTree->end())
+         continue;
+
+      // add this node to its parent's children
+      iter->second->children.push_back(element.second);
+   }
+}
+
+void getChildren(const boost::shared_ptr<ProcessTreeNode>& node,
+                 std::vector<ProcessInfo> *pOutChildren)
+{
+   BOOST_FOREACH(const boost::shared_ptr<ProcessTreeNode>& child, node->children)
+   {
+      pOutChildren->push_back(*child->data.get());
+      getChildren(child, pOutChildren);
+   }
+}
+
+Error getProcesses(std::vector<ProcessInfo> *pOutProcesses)
+{
+   PROCESSENTRY32 processEntry;
+   memset(&processEntry, 0, sizeof(PROCESSENTRY32));
+   processEntry.dwSize = sizeof(PROCESSENTRY32);
+
+   HANDLE hSnap = ::CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
+   if (hSnap == INVALID_HANDLE_VALUE)
+   {
+      return systemError(::GetLastError(), ERROR_LOCATION);
+   }
+
+   if (Process32First(hSnap, &processEntry))
+   {
+      BOOL moreProcesses = TRUE;
+
+      while (moreProcesses)
+      {
+         ProcessInfo process;
+         process.processId = processEntry.th32ProcessID;
+         process.parentProcessId = processEntry.th32ParentProcessID;
+         pOutProcesses->push_back(process);
+
+         moreProcesses = ::Process32Next(hSnap, &processEntry);
+      }
+   }
+
+   return Success();
+}
+
+Error getChildProcesses(std::vector<ProcessInfo> *pOutProcesses)
+{
+   if (!pOutProcesses)
+      return systemError(EINVAL, ERROR_LOCATION);
+
+   // get all processes
+   std::vector<ProcessInfo> processes;
+   Error error = getProcesses(&processes);
+   if (error) return error;
+
+   // build a process tree of the processes
+   ProcessTreeT tree;
+   createProcessTree(processes, &tree);
+
+   // return just the children of this process
+   ProcessTreeT::const_iterator iter = tree.find(::GetCurrentProcessId());
+   if (iter == tree.end())
+      return Success();
+
+   const boost::shared_ptr<ProcessTreeNode>& rootNode = iter->second;
+   getChildren(rootNode, pOutProcesses);
+
+   return Success();
+}
+
+Error terminateChildProcesses()
+{
+   std::vector<ProcessInfo> childProcesses;
+   Error error = getChildProcesses(&childProcesses);
+   if (error)
+      return error;
+
+   BOOST_FOREACH(const ProcessInfo& process, childProcesses)
+   {
+      HANDLE hChildProc = ::OpenProcess(PROCESS_ALL_ACCESS, FALSE, process.processId);
+      if (hChildProc)
+      {
+         if (!::TerminateProcess(hChildProc, 1))
+         {
+            LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+         }
+
+         if (!::CloseHandle(hChildProc))
+         {
+            LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+         }
+      }
+      else
+      {
+         LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+      }
+   }
+
+   // the actual kill is best effort
+   // so return success regardless
+   return Success();
+}
+
+void setHomeToUserProfile(core::system::Options* pChildEnv)
+{
+   std::string userProfile = core::system::getenv(*pChildEnv, "USERPROFILE");
+   core::system::setenv(pChildEnv, "HOME", userProfile);
+}
 
 } // namespace system
 } // namespace core
