@@ -973,20 +973,20 @@ void GwtCallback::openProjectInNewWindow(QString projectFilePath)
 {
    std::vector<std::string> args;
    args.push_back(resolveAliasedPath(projectFilePath).toStdString());
-   launchRStudio(args);
+   pMainWindow_->launchRStudio(args);
 }
 
 void GwtCallback::openSessionInNewWindow(QString workingDirectoryPath)
 {
    workingDirectoryPath = resolveAliasedPath(workingDirectoryPath);
-   core::system::setenv(kRStudioInitialWorkingDir,
-                        workingDirectoryPath.toStdString());
-   launchRStudio();
+   pMainWindow_->launchRStudio(std::vector<std::string>(),
+                               workingDirectoryPath.toStdString());
 }
 
 void GwtCallback::openTerminal(QString terminalPath,
                                QString workingDirectory,
-                               QString extraPathEntries)
+                               QString extraPathEntries,
+                               int shellType)
 {
    // append extra path entries to our path before launching
    std::string path = core::system::getenv("PATH");
@@ -1010,32 +1010,45 @@ void GwtCallback::openTerminal(QString terminalPath,
 
 #elif defined(Q_OS_WIN)
 
-   // git bash
-   if (terminalPath.length() > 0)
+   // TODO: (gary) make these shell type constants shared with
+   // SessionTerminalShell.hpp instead of duplicating them
+   const int GitBash = 1; // Win32: Bash from Windows Git
+   const int WSLBash = 2; // Win32: Windows Services for Linux
+   const int Cmd32 = 3; // Win32: Windows command shell (32-bit)
+   const int Cmd64 = 4; // Win32: Windows command shell (64-bit)
+   const int PS32 = 5; // Win32: PowerShell (32-bit)
+   const int PS64 = 6; // Win32: PowerShell (64-bit)
+
+   if (terminalPath.length() == 0)
    {
-      QStringList args;
+      terminalPath = QString::fromUtf8("cmd.exe");
+      shellType = Cmd32;
+   }
+
+   QStringList args;
+   std::string previousHome = core::system::getenv("HOME");
+
+   switch (shellType)
+   {
+   case GitBash:
+   case WSLBash:
       args.append(QString::fromUtf8("--login"));
       args.append(QString::fromUtf8("-i"));
-      QProcess::startDetached(terminalPath,
-                              args,
-                              resolveAliasedPath(workingDirectory));
-   }
-   else
-   {
+      break;
+
+   default:
       // set HOME to USERPROFILE so msys ssh can find our keys
-      std::string previousHome = core::system::getenv("HOME");
       std::string userProfile = core::system::getenv("USERPROFILE");
       core::system::setenv("HOME", userProfile);
-
-      // run the process
-      QProcess::startDetached(QString::fromUtf8("cmd.exe"),
-                              QStringList(),
-                              resolveAliasedPath(workingDirectory));
-
-      // revert to previous home
-      core::system::setenv("HOME", previousHome);
+      break;
    }
 
+   QProcess::startDetached(terminalPath,
+                           args,
+                           resolveAliasedPath(workingDirectory));
+
+   // revert to previous home
+   core::system::setenv("HOME", previousHome);
 
 #elif defined(Q_OS_LINUX)
 
