@@ -31,6 +31,7 @@
 #include <core/system/Environment.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/FileUtils.hpp>
+#include <core/RegexUtils.hpp>
 #include <core/http/Util.hpp>
 
 #include <r/RExec.hpp>
@@ -41,6 +42,7 @@
 #include <r/RRoutines.hpp>
 #include <r/RInterface.hpp>
 #include <r/RFunctionHook.hpp>
+#include <r/session/RSessionState.hpp>
 #include <r/session/RConsoleActions.hpp>
 #include <r/session/RConsoleHistory.hpp>
 #include <r/session/RClientState.hpp>
@@ -48,7 +50,6 @@
 #include <r/session/RDiscovery.hpp>
 
 #include "RClientMetrics.hpp"
-#include "RSessionState.hpp"
 #include "RRestartContext.hpp"
 #include "REmbedded.hpp"
 
@@ -112,7 +113,7 @@ bool s_initialized = false;
 bool s_suspended = false;
 
 // temporarily suppress output
-bool s_suppressOuput = false;
+bool s_suppressOutput = false;
 
 FilePath rHistoryFilePath()
 {
@@ -649,7 +650,7 @@ bool consoleInputHook(const std::string& prompt,
    // check for user quit invocation
     boost::regex re("^\\s*(q|quit)\\s*\\(.*$");
     boost::smatch match;
-    if (boost::regex_match(input, match, re))
+    if (regex_utils::match(input, match, re))
    {
       if (!s_callbacks.handleUnsavedChanges())
       {
@@ -826,7 +827,7 @@ void RWriteConsoleEx (const char *buf, int buflen, int otype)
 {
    try
    {
-      if (!s_suppressOuput)
+      if (!s_suppressOutput)
       {
          // get output
          std::string output = std::string(buf,buflen);
@@ -1598,8 +1599,7 @@ bool isSuspendable(const std::string& currentPrompt)
 bool suspend(const RSuspendOptions& options,
              const FilePath& suspendedSessionPath,
              bool disableSaveCompression,
-             bool force,
-             int status = EXIT_SUCCESS)
+             bool force)
 {
    // validate that force == true if disableSaveCompression is specified
    // this is because save compression is disabled and the previous options
@@ -1642,7 +1642,7 @@ bool suspend(const RSuspendOptions& options,
    
       // clean up but don't save workspace or runLast because we have
       // been suspended
-      RCleanUp(SA_NOSAVE, status, FALSE);
+      RCleanUp(SA_NOSAVE, options.status, FALSE);
       
       // keep compiler happy (this line will never execute)
       return true;
@@ -1655,11 +1655,10 @@ bool suspend(const RSuspendOptions& options,
 
 bool suspend(bool force, int status)
 {
-   return suspend(RSuspendOptions(),
+   return suspend(RSuspendOptions(status),
                   s_suspendedSessionPath,
                   false,
-                  force,
-                  status);
+                  force);
 }
 
 void suspendForRestart(const RSuspendOptions& options)
@@ -1668,8 +1667,7 @@ void suspendForRestart(const RSuspendOptions& options)
            RestartContext::createSessionStatePath(s_options.scopedScratchPath,
                                                   s_options.sessionPort),
            true,  // disable save compression
-           true,  // force suspend
-           EX_CONTINUE);
+           true);  // force suspend
 }
 
 // set save action
@@ -1800,12 +1798,12 @@ FilePath tempDir()
 
 SuppressOutputInScope::SuppressOutputInScope()
 {
-  s_suppressOuput = true;
+  s_suppressOutput = true;
 }
 
 SuppressOutputInScope::~SuppressOutputInScope()
 {
-   s_suppressOuput = false;
+   s_suppressOutput = false;
 }
 
 } // namespace utils
