@@ -1,7 +1,8 @@
 /*
  * SessionModuleContext.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
+
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -44,6 +45,7 @@ namespace core {
    class FilePath;
    class FileInfo;
    class Settings;
+   class DistributedEvent;
    namespace system {
       class ProcessSupervisor;
       struct ProcessResult;
@@ -65,8 +67,6 @@ namespace session {
 namespace rstudio {
 namespace session {   
 
-class DistributedEvent;
-
 namespace module_context {
 
 enum PackageCompatStatus
@@ -221,6 +221,7 @@ core::Error registerIdleOnlyAsyncRpcMethod(
 core::Error registerRpcMethod(const std::string& name,
                               const core::json::JsonRpcFunction& function);
 
+void registerRpcMethod(const core::json::JsonRpcAsyncMethod& method);
 
 core::Error executeAsync(const core::json::JsonRpcFunction& function,
                          const core::json::JsonRpcRequest& request,
@@ -331,10 +332,13 @@ struct Events : boost::noncopyable
    boost::signal<void(bool)>                 onBackgroundProcessing;
    boost::signal<void(bool)>                 onShutdown;
    boost::signal<void ()>                    onQuit;
+   boost::signal<void ()>                    onDestroyed;
+   boost::signal<void (const std::vector<std::string>&)>
+                                             onLibPathsChanged;
    boost::signal<void (const std::string&)>  onPackageLoaded;
    boost::signal<void ()>                    onPackageLibraryMutated;
    boost::signal<void ()>                    onPreferencesSaved;
-   boost::signal<void (const DistributedEvent&)>
+   boost::signal<void (const core::DistributedEvent&)>
                                              onDistributedEvent;
    boost::signal<void (core::FilePath)>      onPermissionsChanged;
 
@@ -742,6 +746,8 @@ void addViewerHistoryEntry(const ViewerHistoryEntry& entry);
 core::Error recursiveCopyDirectory(const core::FilePath& fromDir,
                                    const core::FilePath& toDir);
 
+bool isSessionTempPath(core::FilePath filePath);
+
 std::string sessionTempDirUrl(const std::string& sessionTempPath);
 
 core::Error uniqueSaveStem(const core::FilePath& directoryPath,
@@ -836,6 +842,8 @@ std::string websiteOutputDir();
 core::FilePath extractOutputFileCreated(const core::FilePath& inputFile,
                                         const std::string& output);
 
+void onBackgroundProcessing(bool isIdle);
+
 } // namespace module_context
 } // namespace session
 } // namespace rstudio
