@@ -71,31 +71,54 @@ namespace {
 
 bool isWithinIgnoredDirectory(const FilePath& filePath)
 {
+   using namespace projects;
+   
    // we only index (and ignore) directories within the current project
-   if (!projects::projectContext().hasProject())
+   if (!projectContext().hasProject())
       return false;
    
-   FilePath projDir = projects::projectContext().directory();
-   for (FilePath parentPath = filePath.parent();
+   FilePath projDir = projectContext().directory();
+   FilePath parentPath = filePath.parent();
+   std::string websiteDir = module_context::websiteOutputDir();
+   bool isPackageProject = projectContext().isPackageProject();
+   
+   // allow plain files living within the 'revdep' folder
+   if (isPackageProject &&
+       parentPath.filename() == "revdep" &&
+       !filePath.isDirectory())
+   {
+      return false;
+   }
+   
+   // look through parent directories recursively to see
+   // if this is an ignored folder
+   // TODO: it would be better to encode this as a filter that
+   // disables traversing of sub-directories we don't want to index
+   for (;
         !parentPath.empty() && parentPath != projDir;
         parentPath = parentPath.parent())
    {
-      // cmake build directory
-      if (parentPath.childPath("cmake_install.cmake").exists())
-         return true;
-      
-      std::string filename = parentPath.filename();
+      std::string parentName = parentPath.filename();
       
       // node_modules
-      if (filename == "node_modules")
+      if (parentName == "node_modules")
+         return true;
+      
+      // websites
+      if (parentName == websiteDir)
          return true;
       
       // packrat
-      if (filename == "packrat" && parentPath.childPath("packrat.lock").exists())
+      if (parentName == "packrat" &&
+          parentPath.childPath("packrat.lock").exists())
+         return true;
+      
+      // cmake build directory
+      if (parentPath.childPath("cmake_install.cmake").exists())
          return true;
 
-      // websites
-      if (filename == module_context::websiteOutputDir())
+      // revdep sub-directories
+      if (isPackageProject && parentName == "revdep")
          return true;
    }
    
@@ -892,7 +915,7 @@ private:
                ext == ".h" || ext == ".hpp" ||
                ext == ".c" || ext == ".cpp" ||
                ext == ".json" || ext == ".tex" ||
-               ext == ".scala" ||
+               ext == ".toml" || ext == ".scala" ||
                filename == "DESCRIPTION" ||
                filename == "NAMESPACE" ||
                filename == "README" ||
@@ -1675,6 +1698,15 @@ bool namespaceIsPackage(const std::string& namespaceName,
    }
 }
 
+bool findData(const std::string& name)
+{
+   std::string found;
+   Error error = r::exec::RFunction(".rs.findGlobalData", name).call(&found);
+   if (error)
+      LOG_ERROR(error);
+   return !found.empty();
+}
+
 bool findFunction(const std::string& name,
                   const std::string& fromWhere,
                   std::string* pNamespaceName)
@@ -1772,7 +1804,7 @@ public:
    {
       boost::regex pattern("^([^{]+)\\{([^}]+)\\}$");
       boost::smatch match;
-      if (boost::regex_search(name_, match, pattern))
+      if (regex_utils::search(name_, match, pattern))
       {
          // read method name
          methodName_ = match[1];
@@ -2079,7 +2111,7 @@ Error guessFunctionToken(const std::string& line,
    // see if it has a namespace qualifier
    boost::regex pattern("^([^:]+):{2,3}([^:]+)$");
    boost::smatch match;
-   if (boost::regex_search(token, match, pattern))
+   if (regex_utils::search(token, match, pattern))
    {
       pToken->package = match[1];
       pToken->name = match[2];
@@ -2110,20 +2142,21 @@ Error getFunctionDefinition(const json::JsonRpcRequest& request,
 
    // default return value is null function name (indicating no results)
    json::Object defJson;
-   defJson["function_name"] = json::Value();
+   defJson["name"] = json::Value();
 
    // if there was a package then we go straight to the search path
    if (!token.package.empty())
    {
-      defJson["function_name"] = token.name;
-      defJson["search_path_definition"] = createFunctionDefinition(token);
+      defJson["name"] = token.name;
+      defJson["type"] = "search_path_function";
+      defJson["data"] = createFunctionDefinition(token);
    }
 
    // if we got a name token then search the code
    else if (!token.name.empty())
    {
       // discovered a token so we have at least a function name to return
-      defJson["function_name"] = token.name;
+      defJson["name"] = token.name;
 
       // find in source database then in project index
       std::set<std::string> contexts;
@@ -2136,15 +2169,19 @@ Error getFunctionDefinition(const json::JsonRpcRequest& request,
       if (found)
       {
          // return full path to file
+         json::Object fileDef;
          FilePath srcFilePath = module_context::resolveAliasedPath(
                                                       sourceItem.context());
-         defJson["file"] = module_context::createFileSystemItem(srcFilePath);
+         fileDef["file"] = module_context::createFileSystemItem(srcFilePath);
 
          // return location in file
          json::Object posJson;
          posJson["line"] = sourceItem.line();
          posJson["column"] = sourceItem.column();
-         defJson["position"] = posJson;
+         fileDef["position"] = posJson;
+
+         defJson["type"] = "file_function";
+         defJson["data"] = fileDef;
       }
       // didn't find the file, check the search path
       else
@@ -2153,10 +2190,34 @@ Error getFunctionDefinition(const json::JsonRpcRequest& request,
          std::string namespaceName;
          if (findFunction(token.name, "", &namespaceName))
          {
-            defJson["search_path_definition"] =
-                              createFunctionDefinition(token.name,
+            defJson["type"] = "search_path_function";
+            defJson["data"] = createFunctionDefinition(token.name,
                                                        namespaceName);
          }
+         else if (findData(token.name))
+         {
+            defJson["type"] = "data";
+            defJson["data"] = "";
+         }
+         else
+         {
+            // try finding in the completion database
+            using namespace r_util;
+            
+            BOOST_REVERSE_FOREACH(
+                     const std::string& package,
+                     RSourceIndex::getAllInferredPackages())
+            {
+               PackageInformation information = RSourceIndex::getPackageInformation(package);
+               if (information.functionInfo.count(token.name))
+               {
+                  defJson["type"] = "search_path_function";
+                  defJson["data"] = createFunctionDefinition(token.name, package);
+                  break;
+                  
+               }
+            }
+         }
       }
    }
 
