@@ -994,6 +994,6 @@ Error initialize()
    
 } // namespace environment
 } // namespace modules
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
