@@ -885,6 +885,6 @@ Error initialize()
 } // namespace compile_pdf
 } // namespace tex
 } // namespace modules
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
