@@ -15,6 +15,8 @@
 
 #include <session/projects/SessionProjects.hpp>
 
+#include <boost/algorithm/string/trim.hpp>
+
 #include <core/FilePath.hpp>
 #include <core/Settings.hpp>
 #include <core/Exec.hpp>
@@ -26,6 +28,7 @@
 
 #include <session/SessionModuleContext.hpp>
 #include <session/SessionUserSettings.hpp>
+#include <session/SessionProjectTemplate.hpp>
 #include <session/SessionScopes.hpp>
 
 #include <session/projects/ProjectsSettings.hpp>
@@ -125,6 +128,32 @@ Error getNewProjectContext(const json::JsonRpcRequest& request,
    return Success();
 }
 
+Error initializeProjectFromTemplate(const FilePath& projectFilePath,
+                                    const json::Value& projectTemplateOptions)
+{
+   using namespace modules::projects::templates;
+   using namespace r::exec;
+   Error error;
+
+   json::Object descriptionJson;
+   json::Object inputsJson;
+   error = json::readObject(projectTemplateOptions.get_obj(),
+                            "description", &descriptionJson,
+                            "inputs", &inputsJson);
+   if (error)
+      return error;
+   
+   FilePath projectPath = projectFilePath.parent();
+   
+   return r::exec::RFunction(".rs.initializeProjectFromTemplate")
+         .addParam(string_utils::utf8ToSystem(projectFilePath.absolutePath()))
+         .addParam(string_utils::utf8ToSystem(projectPath.absolutePath()))
+         .addParam(descriptionJson)
+         .addParam(inputsJson)
+         .call();
+
+}
+
 Error createProject(const json::JsonRpcRequest& request,
                     json::JsonRpcResponse* pResponse)
 {
@@ -132,14 +161,17 @@ Error createProject(const json::JsonRpcRequest& request,
    std::string projectFile;
    json::Value newPackageJson;
    json::Value newShinyAppJson;
+   json::Value projectTemplateOptions;
    Error error = json::readParams(request.params,
                                   &projectFile,
                                   &newPackageJson,
-                                  &newShinyAppJson);
+                                  &newShinyAppJson,
+                                  &projectTemplateOptions);
    if (error)
       return error;
    FilePath projectFilePath = module_context::resolveAliasedPath(projectFile);
 
+   // Shiny application
    if (!newShinyAppJson.is_null())
    {
       // error if the shiny app dir already exists
@@ -152,53 +184,74 @@ Error createProject(const json::JsonRpcRequest& request,
       if (error)
          return error;
 
-      // copy ui.R and server.R into the project
-      const char * const kUI = "ui.R";
-      const char * const kServer = "server.R";
-      std::string shinyVer;
-      if (module_context::isPackageVersionInstalled("shiny", "0.9"))
-         shinyVer = "shiny-0.9";
-      else
-         shinyVer = "shiny";
-      FilePath shinyDir = session::options().rResourcesPath().childPath(
-                                                     "templates/" + shinyVer);
-      error = shinyDir.childPath(kUI).copy(appDir.childPath(kUI));
-      if (error)
-         LOG_ERROR(error);
-      error = shinyDir.childPath(kServer).copy(appDir.childPath(kServer));
+      // copy app.R into the project
+      FilePath shinyDir = session::options().rResourcesPath()
+            .childPath("templates/shiny");
+      
+      error = shinyDir.childPath("app.R").copy(appDir.childPath("app.R"));
       if (error)
          LOG_ERROR(error);
 
       // add first run actions for the source files
-      addFirstRunDoc(projectFilePath, kUI);
-      addFirstRunDoc(projectFilePath, kServer);
+      addFirstRunDoc(projectFilePath, "app.R");
 
       // create the project file
       return r_util::writeProjectFile(projectFilePath,
                                       ProjectContext::buildDefaults(),
                                       ProjectContext::defaultConfig());
    }
-
-   // default project
-   else
+   
+   // if we have a custom project template, call that first
+   if (!projectTemplateOptions.is_null() &&
+       json::isType<json::Object>(projectTemplateOptions))
    {
-      // create the project directory if necessary
-      error = projectFilePath.parent().ensureDirectory();
+      Error error = initializeProjectFromTemplate(projectFilePath, projectTemplateOptions);
       if (error)
          return error;
-
-      // create the project file
-      if (!projectFilePath.exists())
-      {
-         return r_util::writeProjectFile(projectFilePath,
-                                         ProjectContext::buildDefaults(),
-                                         ProjectContext::defaultConfig());
-      }
-      else
-      {
-         return Success();
-      }
    }
+   
+   // default project scaffolding
+   error = projectFilePath.parent().ensureDirectory();
+   if (error)
+      return error;
+
+   // create the project file
+   if (!projectFilePath.exists())
+   {
+      error = r_util::writeProjectFile(projectFilePath,
+                                       ProjectContext::buildDefaults(),
+                                       ProjectContext::defaultConfig());
+      if (error)
+         return error;
+   }
+   
+   return Success();
+}
+
+Error createProjectFile(const json::JsonRpcRequest& request,
+                        json::JsonRpcResponse* pResponse)
+{
+   using namespace projects;
+   
+   std::string projFile;
+   Error error = json::readParams(request.params, &projFile);
+   if (error)
+      return error;
+   
+   FilePath projPath = module_context::resolveAliasedPath(projFile);
+   if (!projPath.exists())
+   {
+      Error error = r_util::writeProjectFile(
+               projPath,
+               ProjectContext::buildDefaults(),
+               ProjectContext::defaultConfig());
+      
+      if (error)
+         LOG_ERROR(error);
+   }
+   
+   pResponse->setResult(projPath.exists());
+   return Success();
 }
 
 json::Object projectConfigJson(const r_util::RProjectConfig& config)
@@ -234,6 +287,8 @@ json::Object projectConfigJson(const r_util::RProjectConfig& config)
    configJson["website_path"] = config.websitePath;
    configJson["custom_script_path"] = config.customScriptPath;
    configJson["tutorial_path"] = config.tutorialPath;
+   configJson["quit_child_processes_on_exit"] = config.quitChildProcessesOnExit;
+
    return configJson;
 }
 
@@ -411,10 +466,35 @@ Error writeProjectOptions(const json::JsonRpcRequest& request,
                     "default_encoding", &(config.encoding),
                     "default_sweave_engine", &(config.defaultSweaveEngine),
                     "default_latex_program", &(config.defaultLatexProgram),
-                    "root_document", &(config.rootDocument));
+                    "root_document", &(config.rootDocument),
+                    "quit_child_processes_on_exit", &(config.quitChildProcessesOnExit));
    if (error)
       return error;
 
+   // get the default_open_files options
+   // this is currently not writeable by the UI
+   // so we need to make sure we persist the current value
+   bool providedDefaults;
+   std::string userErrMsg;
+   r_util::RProjectConfig existingConfig;
+   error = r_util::readProjectFile(s_projectContext.file(),
+                                         ProjectContext::defaultConfig(),
+                                         ProjectContext::buildDefaults(),
+                                         &existingConfig,
+                                         &providedDefaults,
+                                         &userErrMsg);
+   if (error)
+   {
+      LOG_ERROR(error);
+   }
+   else
+   {
+      if (!existingConfig.defaultOpenDocs.empty())
+      {
+         config.defaultOpenDocs = existingConfig.defaultOpenDocs;
+      }
+   }
+
    error = json::readObject(
                     configJson,
                     "auto_append_newline", &(config.autoAppendNewline),
@@ -574,7 +654,7 @@ FilePath resolveProjectSwitch(const std::string& projectPath)
 }  // anonymous namespace
 
 
-void startup()
+void startup(const std::string& firstProjectPath)
 {
    // register suspend handler
    using namespace module_context;
@@ -586,6 +666,9 @@ void startup()
    // alias some project context data
    projects::ProjectsSettings projSettings(options().userScratchPath());
    std::string nextSessionProject = projSettings.nextSessionProject();
+   if (!firstProjectPath.empty())
+      nextSessionProject = firstProjectPath;
+
    std::string switchToProject = projSettings.switchToProjectPath();
    FilePath lastProjectPath = projSettings.lastProjectPath();
 
@@ -651,10 +734,11 @@ void startup()
 
    // if we have a project file path then try to initialize the
    // project context (show a warning to the user if we can't)
+   bool isNewProject = false;
    if (!projectFilePath.empty())
    {
       std::string userErrMsg;
-      Error error = s_projectContext.startup(projectFilePath, &userErrMsg);
+      Error error = s_projectContext.startup(projectFilePath, &userErrMsg, &isNewProject);
       if (error)
       {
          // log the error
@@ -671,6 +755,26 @@ void startup()
          module_context::enqueClientEvent(event);
       }
    }
+
+   // add default open docs if specified in the project
+   // and the project has never been opened before
+   std::string defaultOpenDocs = projectContext().config().defaultOpenDocs;
+   if (!defaultOpenDocs.empty() && isNewProject)
+   {
+      std::vector<std::string> docs;
+      boost::algorithm::split(docs, defaultOpenDocs, boost::is_any_of(":"));
+
+      BOOST_FOREACH(std::string& doc, docs)
+      {
+         boost::algorithm::trim(doc);
+
+         FilePath docPath = projectContext().directory().complete(doc);
+         if (docPath.exists())
+         {
+            addFirstRunDoc(projectFilePath, doc);
+         }
+      }
+   }
 }
 
 SEXP rs_writeProjectFile(SEXP projectFilePathSEXP)
@@ -689,28 +793,53 @@ SEXP rs_writeProjectFile(SEXP projectFilePathSEXP)
             r::sexp::create(true, &protect);
 }
 
-SEXP rs_addFirstRunDoc(SEXP projectFileAbsolutePathSEXP, SEXP docRelativePathSEXP)
+SEXP rs_addFirstRunDoc(SEXP projectFileAbsolutePathSEXP, SEXP docRelativePathsSEXP)
 {
    std::string projectFileAbsolutePath = r::sexp::asString(projectFileAbsolutePathSEXP);
    const FilePath projectFilePath(projectFileAbsolutePath);
    
-   std::string docRelativePath = r::sexp::asString(docRelativePathSEXP);
+   std::vector<std::string> docRelativePaths;
+   bool success = r::sexp::fillVectorString(docRelativePathsSEXP, &docRelativePaths);
+   if (!success)
+      return R_NilValue;
+   
+   BOOST_FOREACH(const std::string& path, docRelativePaths)
+   {
+      addFirstRunDoc(projectFilePath, path);
+   }
+   
+   return R_NilValue;
+}
+
+SEXP rs_requestOpenProject(SEXP projectFileSEXP, SEXP newSessionSEXP)
+{
+   std::string projectFile = r::sexp::asString(projectFileSEXP);
+   bool newSession = r::sexp::asLogical(newSessionSEXP);
+   
+   // opening projects in a new session is only supported in desktop, RSP
+   if (newSession &&
+       options().programMode() == kSessionProgramModeServer &&
+       !options().multiSession())
+   {
+      newSession = false;
+   }
+   
+   json::Object data;
+   data["project_file"] = projectFile;
+   data["new_session"] = newSession;
+   
+   ClientEvent event(client_events::kRequestOpenProject, data);
+   module_context::enqueClientEvent(event);
    
-   addFirstRunDoc(projectFilePath, docRelativePath);
    return R_NilValue;
 }
 
 Error initialize()
 {
-   r::routines::registerCallMethod(
-            "rs_writeProjectFile",
-            (DL_FUNC) rs_writeProjectFile,
-            1);
-   
-   r::routines::registerCallMethod(
-            "rs_addFirstRunDoc",
-            (DL_FUNC) rs_addFirstRunDoc,
-            2);
+   // register R methods
+   RS_REGISTER_CALL_METHOD(rs_writeProjectFile, 1);
+   RS_REGISTER_CALL_METHOD(rs_addFirstRunDoc, 2);
+   RS_REGISTER_CALL_METHOD(rs_requestOpenProject, 2);
    
    // call project-context initialize
    Error error = s_projectContext.initialize();
@@ -734,6 +863,7 @@ Error initialize()
       (bind(registerRpcMethod, "get_new_project_context", getNewProjectContext))
       (bind(registerRpcMethod, "get_project_file_path", getProjectFilePath))
       (bind(registerRpcMethod, "create_project", createProject))
+      (bind(registerRpcMethod, "create_project_file", createProjectFile))
       (bind(registerRpcMethod, "read_project_options", readProjectOptions))
       (bind(registerRpcMethod, "write_project_options", writeProjectOptions))
       (bind(registerRpcMethod, "write_project_vcs_options", writeProjectVcsOptions))
@@ -765,6 +895,6 @@ json::Array websiteOutputFormatsJson()
 }
 
 } // namespace projects
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
