@@ -125,10 +125,10 @@ var MarkdownHighlightRules = function() {
             regex : "^([ ]{0,3}\\[)([^\\]]+)(\\]:\\s*)([^ ]+)(\\s*(?:[\"][^\"]+[\"])?(\\s*))$"
         }, { // link by reference
             token : ["text", "keyword", "text", "constant", "text"],
-            regex : "(\\[)(" + escaped("]") + ")(\\]\s*\\[)("+ escaped("]") + ")(\\])"
+            regex : "((?:[^^]|^)\\[)(" + escaped("]") + ")(\\]\s*\\[)("+ escaped("]") + ")(\\])"
         }, { // link by url
             token : ["text", "keyword", "text", "markup.href", "string", "text"],
-            regex : "(\\[)(" +                                    // [
+            regex : "((?:[^^]|^)\\[)(" +                          // [
                 escaped("]") +                                    // link text
                 ")(\\]\\()"+                                      // ](
                 '((?:[^\\)\\s\\\\]|\\\\.|\\s(?=[^"]))*)' +        // href
@@ -148,9 +148,9 @@ var MarkdownHighlightRules = function() {
             emphasisUnderscore
         ],
        
-        start: [{
+        "start": [{
            token : "empty_line",
-           regex : '^$',
+           regex : '^\\s*$',
            next: "allowBlock"
         }, { // inline r code
             token : "support.function.inline_r_chunk",
@@ -193,12 +193,14 @@ var MarkdownHighlightRules = function() {
             token : "support.function",
             regex : "^\\s*```\\s*\\S*(?:{.*?\\})?\\s*$",
             next  : "githubblock"
+        }, { // ioslides-style bullet
+            token : "string.blockquote",
+            regex : "^\\s*>\\s*(?=[-])"
         }, { // block quote
             token : "string.blockquote",
-            regex : "^\\s*>\\s*(?:[*+-]|\\d+\\.)?\\s+",
+            regex : "^\\s*>\\s*",
             next  : "blockquote"
-        },
-        { // reference
+        }, { // reference
             token : ["text", "constant", "text", "url", "string", "text"],
             regex : "^([ ]{0,3}\\[)([^\\]]+)(\\]:\\s*)([^ ]+)(\\s*(?:[\"][^\"]+[\"])?(\\s*))$"
         }, { // link by reference
@@ -252,7 +254,7 @@ var MarkdownHighlightRules = function() {
             regex : "-?\\@[\\w\\d-]+"
         }, {
             token : "text",
-            regex : "[^\\*_%$`\\[#<>{}\\\\@\\s]+"
+            regex : "[^\\*_%$`\\[#<>{}\\\\@\\s!]+"
         }, {
             token : "text",
             regex : "\\\\"
@@ -290,6 +292,10 @@ var MarkdownHighlightRules = function() {
            token : "support.function",
            regex : "^ {4}.+",
            next : "allowBlock"
+        }, {
+            token : "empty_line",
+            regex : "^\\s*$",
+            next  : "allowBlock"
         }, {
            token : "empty",
            regex : "",
@@ -323,17 +329,46 @@ var MarkdownHighlightRules = function() {
             defaultToken : "text" //do not use markup.list to allow stling leading `*` differntly
         }],
 
-        "blockquote" : [ { // Blockquotes only escape on blank lines.
+        "blockquote" : [{ // Blockquotes only escape on blank lines.
             token : "empty_line",
             regex : "^\\s*$",
             next  : "start"
-        }, { // block quote
-            token : "string.blockquote",
-            regex : "^\\s*>\\s*(?:[*+-]|\\d+\\.)?\\s+",
-            next  : "blockquote"
         }, {
-            include : "basic", noEscape: true
+            token : "constant.language.escape",
+            regex : /\\[\\`*_{}\[\]()#+\-.!]/
+        }, { // inline r code
+            token : "support.function.inline_r_chunk",
+            regex : "`r (?:.*?[^`])`"
+        }, { // code span `
+            token : ["support.function", "support.function", "support.function"],
+            regex : "(`+)(.*?[^`])(\\1)"
+        }, { // reference
+            token : ["text", "constant", "text", "url", "string", "text"],
+            regex : "^([ ]{0,3}\\[)([^\\]]+)(\\]:\\s*)([^ ]+)(\\s*(?:[\"][^\"]+[\"])?(\\s*))$"
+        }, { // link by reference
+            token : ["text", "keyword", "text", "constant", "text"],
+            regex : "((?:[^^]|^)\\[)(" + escaped("]") + ")(\\]\s*\\[)("+ escaped("]") + ")(\\])"
+        }, { // link by url
+            token : ["text", "keyword", "text", "markup.href", "string", "text"],
+            regex : "((?:[^^]|^)\\[)(" +                          // [
+                escaped("]") +                                    // link text
+                ")(\\]\\()"+                                      // ](
+                '((?:[^\\)\\s\\\\]|\\\\.|\\s(?=[^"]))*)' +        // href
+                '(\\s*"' +  escaped('"') + '"\\s*)?' +            // "title"
+                "(\\))"                                           // )
         }, {
+            token : ["text", "keyword", "text"],
+            regex : "(<)("+
+                "(?:https?|ftp|dict):[^'\">\\s]+"+
+                "|"+
+                "(?:mailto:)?[-.\\w]+\\@[-a-z0-9]+(?:\\.[-a-z0-9]+)*\\.[a-z]+"+
+                ")(>)"
+        },
+            strongStars,
+            strongUnderscore,
+            emphasisStars,
+            emphasisUnderscore,
+        {
             defaultToken : "string.blockquote"
         }],
 
