@@ -1,7 +1,7 @@
 /*
  * Stopwatch.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,8 +16,13 @@ package org.rstudio.core.client;
 
 public class Stopwatch
 {
-   public Stopwatch()
+   /**
+    * Track how long something takes, you know, like a Stopwatch.
+    * @param debugLog Show timing via Debug.log
+    */
+   public Stopwatch(boolean debugLog)
    {
+      debugLog_ = debugLog;
       reset();
    }
 
@@ -26,11 +31,14 @@ public class Stopwatch
       startTime_ = System.currentTimeMillis();
    }
 
-   public void mark(String label)
+   public long mark(String label)
    {
       long stopTime = System.currentTimeMillis();
-      Debug.log("[Stopwatch] " + label + ": " + (stopTime - startTime_) + " ms");
+      if (debugLog_)
+         Debug.log("[Stopwatch] " + label + ": " + (stopTime - startTime_) + " ms");
+      return stopTime - startTime_;
    }
 
    private long startTime_;
+   private boolean debugLog_;
 }
