@@ -1,7 +1,7 @@
 /*
  * Application.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,6 +17,7 @@ package org.rstudio.studio.client.application;
 
 import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.RunAsyncCallback;
+import com.google.gwt.dom.client.Document;
 import com.google.gwt.dom.client.Element;
 import com.google.gwt.dom.client.FormElement;
 import com.google.gwt.dom.client.InputElement;
@@ -40,6 +41,7 @@ import com.google.inject.Singleton;
 
 import org.rstudio.core.client.Barrier;
 import org.rstudio.core.client.BrowseCap;
+import org.rstudio.core.client.CommandWithArg;
 import org.rstudio.core.client.Debug;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.Barrier.Token;
@@ -57,6 +59,7 @@ import org.rstudio.studio.client.application.model.InvalidSessionInfo;
 import org.rstudio.studio.client.application.model.ProductInfo;
 import org.rstudio.studio.client.application.model.SessionSerializationAction;
 import org.rstudio.studio.client.application.ui.AboutDialog;
+import org.rstudio.studio.client.application.ui.RStudioThemes;
 import org.rstudio.studio.client.application.ui.RequestLogVisualization;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.SimpleRequestCallback;
@@ -143,6 +146,7 @@ public class Application implements ApplicationEventHandlers
       events.addHandler(ServerOfflineEvent.TYPE, this);
       events.addHandler(InvalidSessionEvent.TYPE, this);
       events.addHandler(SwitchToRVersionEvent.TYPE, this);
+      events.addHandler(ThemeChangedEvent.TYPE, this);
       
       // register for uncaught exceptions
       uncaughtExHandler.register();
@@ -151,8 +155,11 @@ public class Application implements ApplicationEventHandlers
    public void go(final RootLayoutPanel rootPanel, 
                   final Command dismissLoadingProgress)
    {
+      rootPanel_ = rootPanel;
+
       Widget w = view_.getWidget();
       rootPanel.add(w);
+
       rootPanel.setWidgetTopBottom(w, 0, Style.Unit.PX, 0, Style.Unit.PX);
       rootPanel.setWidgetLeftRight(w, 0, Style.Unit.PX, 0, Style.Unit.PX);
 
@@ -611,6 +618,14 @@ public class Application implements ApplicationEventHandlers
       view_.showSessionAbendWarning();
    }
    
+   @Override
+   public void onThemeChanged(ThemeChangedEvent event)
+   {
+      RStudioThemes.initializeThemes(uiPrefs_.get(),
+                                     Document.get(),
+                                     rootPanel_.getElement());
+   }
+   
    private void verifyAgreement(SessionInfo sessionInfo,
                               final Operation verifiedOperation)
    {
@@ -703,6 +718,22 @@ public class Application implements ApplicationEventHandlers
    
    private void initializeWorkbench()
    {
+      CommandWithArg<String> changedTheme = new CommandWithArg<String>()
+      {
+         public void execute(String theme)
+         {
+            RStudioThemes.initializeThemes(uiPrefs_.get(),
+                                           Document.get(),
+                                           rootPanel_.getElement());
+
+            events_.fireEventToAllSatellites(new ThemeChangedEvent(theme));
+         }
+      };
+
+      uiPrefs_.get().getFlatTheme().bind(changedTheme);
+      uiPrefs_.get().theme().bind(changedTheme);
+      changedTheme.execute(uiPrefs_.get().getFlatTheme().getValue());
+
       pAceThemes_.get();
 
       // subscribe to ClientDisconnected event (wait to do this until here
@@ -720,10 +751,23 @@ public class Application implements ApplicationEventHandlers
 
       // disable commands
       SessionInfo sessionInfo = session_.getSessionInfo();
+
+      if (BrowseCap.isWindowsDesktop())
+      {
+         commands_.interruptTerminal().remove();
+      }
+
       if (!sessionInfo.getAllowShell())
       {
          commands_.showShellDialog().remove();
+         removeTerminalCommands();
       }
+      
+      if (!sessionInfo.getAllowFullUI())
+      {
+         removeProjectCommands();
+      }
+
       if (!sessionInfo.getAllowPackageInstallation())
       {
          commands_.installPackage().remove();
@@ -737,6 +781,10 @@ public class Application implements ApplicationEventHandlers
       {
          commands_.exportFiles().remove();
       }
+      if (!sessionInfo.getAllowFileUploads())
+      {
+         commands_.uploadFile().remove();
+      }
       
       // disable external publishing if requested
       if (!SessionUtils.showExternalPublishUi(session_, uiPrefs_.get()))
@@ -758,7 +806,7 @@ public class Application implements ApplicationEventHandlers
          commands_.importDatasetFromFile().remove();
          commands_.importDatasetFromURL().remove();
          
-         commands_.importDatasetFromCSV().setVisible(false);
+         commands_.importDatasetFromCsvUsingReadr().setVisible(false);
          commands_.importDatasetFromSAV().setVisible(false);
          commands_.importDatasetFromSAS().setVisible(false);
          commands_.importDatasetFromStata().setVisible(false);
@@ -771,7 +819,7 @@ public class Application implements ApplicationEventHandlers
             String rVersion = sessionInfo.getRVersionsInfo().getRVersion();
             if (ApplicationUtils.compareVersions(rVersion, "3.0.2") >= 0)
             {
-               commands_.importDatasetFromCSV().setVisible(true);
+               commands_.importDatasetFromCsvUsingReadr().setVisible(true);
             }
             if (ApplicationUtils.compareVersions(rVersion, "3.1.0") >= 0)
             {
@@ -803,7 +851,9 @@ public class Application implements ApplicationEventHandlers
       }
       else
       {
-         commands_.importDatasetFromCSV().remove();
+         commands_.importDatasetFromCsv().remove();
+         commands_.importDatasetFromCsvUsingBase().remove();
+         commands_.importDatasetFromCsvUsingReadr().remove();
          commands_.importDatasetFromSAV().remove();
          commands_.importDatasetFromSAS().remove();
          commands_.importDatasetFromStata().remove();
@@ -950,6 +1000,46 @@ public class Application implements ApplicationEventHandlers
       navigateWindowTo("auth-sign-in");
    }
    
+   private void removeTerminalCommands()
+   {
+      commands_.newTerminal().remove();
+      commands_.activateTerminal().remove();
+      commands_.renameTerminal().remove();
+      commands_.closeTerminal().remove();
+      commands_.clearTerminalScrollbackBuffer().remove();
+      commands_.previousTerminal().remove();
+      commands_.nextTerminal().remove();
+      commands_.showTerminalInfo().remove();
+      commands_.interruptTerminal().remove();
+      commands_.sendTerminalToEditor().remove();
+      commands_.sendToTerminal().remove();
+   }
+
+   private void removeProjectCommands()
+   {
+      commands_.openProject().remove();
+      commands_.newProject().remove();
+      commands_.closeProject().remove();
+      commands_.openProjectInNewWindow().remove();
+      commands_.clearRecentProjects().remove();
+      commands_.quitSession().remove();
+      commands_.projectMru0().remove();
+      commands_.projectMru1().remove();
+      commands_.projectMru2().remove();
+      commands_.projectMru3().remove();
+      commands_.projectMru4().remove();
+      commands_.projectMru5().remove();
+      commands_.projectMru6().remove();
+      commands_.projectMru7().remove();
+      commands_.projectMru8().remove();
+      commands_.projectMru9().remove();
+      commands_.projectMru10().remove();
+      commands_.projectMru11().remove();
+      commands_.projectMru12().remove();
+      commands_.projectMru13().remove();
+      commands_.projectMru14().remove();
+    }
+
    private final ApplicationView view_ ;
    private final GlobalDisplay globalDisplay_ ;
    private final EventBus events_;
@@ -969,4 +1059,5 @@ public class Application implements ApplicationEventHandlers
    private final String CSRF_TOKEN_FIELD = "csrf-token";
 
    private ClientStateUpdater clientStateUpdaterInstance_;
+   private RootLayoutPanel rootPanel_;
 }
