@@ -1,7 +1,7 @@
 /*
  * ShellWidget.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,22 +14,11 @@
  */
 package org.rstudio.studio.client.common.shell;
 
+import java.util.List;
 import java.util.Map;
 import java.util.TreeMap;
 
-import com.google.gwt.core.client.Scheduler;
-import com.google.gwt.core.client.Scheduler.RepeatingCommand;
-import com.google.gwt.core.client.Scheduler.ScheduledCommand;
-import com.google.gwt.dom.client.Document;
-import com.google.gwt.dom.client.Element;
-import com.google.gwt.dom.client.Node;
-import com.google.gwt.dom.client.SpanElement;
-import com.google.gwt.dom.client.Style.Unit;
-import com.google.gwt.dom.client.Text;
-import com.google.gwt.event.dom.client.*;
-import com.google.gwt.event.shared.HandlerRegistration;
-import com.google.gwt.user.client.ui.*;
-
+import org.rstudio.core.client.ConsoleOutputWriter;
 import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.TimeBufferedCommand;
@@ -54,6 +43,30 @@ import org.rstudio.studio.client.workbench.views.source.editors.text.events.Curs
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.CursorChangedHandler;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.PasteEvent;
 
+import com.google.gwt.core.client.Scheduler;
+import com.google.gwt.core.client.Scheduler.RepeatingCommand;
+import com.google.gwt.core.client.Scheduler.ScheduledCommand;
+import com.google.gwt.dom.client.Document;
+import com.google.gwt.dom.client.Element;
+import com.google.gwt.dom.client.SpanElement;
+import com.google.gwt.dom.client.Style.Unit;
+import com.google.gwt.event.dom.client.ClickEvent;
+import com.google.gwt.event.dom.client.ClickHandler;
+import com.google.gwt.event.dom.client.FocusEvent;
+import com.google.gwt.event.dom.client.FocusHandler;
+import com.google.gwt.event.dom.client.KeyCodes;
+import com.google.gwt.event.dom.client.KeyDownEvent;
+import com.google.gwt.event.dom.client.KeyDownHandler;
+import com.google.gwt.event.dom.client.KeyPressHandler;
+import com.google.gwt.event.dom.client.KeyUpHandler;
+import com.google.gwt.event.shared.HandlerRegistration;
+import com.google.gwt.user.client.ui.Composite;
+import com.google.gwt.user.client.ui.DockPanel;
+import com.google.gwt.user.client.ui.HTML;
+import com.google.gwt.user.client.ui.RequiresResize;
+import com.google.gwt.user.client.ui.VerticalPanel;
+import com.google.gwt.user.client.ui.Widget;
+
 public class ShellWidget extends Composite implements ShellDisplay,
                                                       RequiresResize,
                                                       ConsoleError.Observer
@@ -65,12 +78,12 @@ public class ShellWidget extends Composite implements ShellDisplay,
       
       SelectInputClickHandler secondaryInputHandler = new SelectInputClickHandler();
 
-      output_ = new PreWidget();
-      output_.setStylePrimaryName(styles_.output());
-      output_.addClickHandler(secondaryInputHandler);
+      output_ = new ConsoleOutputWriter();
+      output_.getWidget().setStylePrimaryName(styles_.output());
+      output_.getWidget().addClickHandler(secondaryInputHandler);
       ElementIds.assignElementId(output_.getElement(), 
                                  ElementIds.CONSOLE_OUTPUT);
-      output_.addPasteHandler(secondaryInputHandler);
+      output_.getWidget().addPasteHandler(secondaryInputHandler);
 
       pendingInput_ = new PreWidget();
       pendingInput_.setStyleName(styles_.output());
@@ -158,18 +171,18 @@ public class ShellWidget extends Composite implements ShellDisplay,
       inputLine_.setCellWidth(input_.asWidget(), "100%");
       inputLine_.setWidth("100%");
 
-      verticalPanel_ = new VerticalPanel() ;
+      verticalPanel_ = new VerticalPanel();
       verticalPanel_.setStylePrimaryName(styles_.console());
       verticalPanel_.addStyleName("ace_text-layer");
       verticalPanel_.addStyleName("ace_line");
       FontSizer.applyNormalFontSize(verticalPanel_);
-      verticalPanel_.add(output_) ;
-      verticalPanel_.add(pendingInput_) ;
-      verticalPanel_.add(inputLine_) ;
-      verticalPanel_.setWidth("100%") ;
+      verticalPanel_.add(output_.getWidget());
+      verticalPanel_.add(pendingInput_);
+      verticalPanel_.add(inputLine_);
+      verticalPanel_.setWidth("100%");
 
-      scrollPanel_ = new ClickableScrollPanel() ;
-      scrollPanel_.setWidget(verticalPanel_) ;
+      scrollPanel_ = new ClickableScrollPanel();
+      scrollPanel_.setWidget(verticalPanel_);
       scrollPanel_.addStyleName("ace_editor");
       scrollPanel_.addStyleName("ace_scroller");
       scrollPanel_.addClickHandler(secondaryInputHandler);
@@ -249,20 +262,21 @@ public class ShellWidget extends Composite implements ShellDisplay,
    public void consoleWriteError(final String error)
    {
       clearPendingInput();
-      output(error, getErrorClass(), false);
+      output(error, getErrorClass(), true /*isError*/, false /*ignoreLineCount*/);
 
-      // Pick up the last element emitted to the console. If we get extended
-      // information for this error, we'll need to swap out the simple error
-      // element for the extended error element. 
-      Element outputElement = output_.getElement();
-      Node errorNode = outputElement.getChild(
-            outputElement.getChildCount() - 1);
-      if (clearErrors_)
+      // Pick up the elements emitted to the console by this call. If we get 
+      // extended information for this error, we'll need to swap out the simple 
+      // error elements for the extended error element. 
+      List<Element> newElements = output_.getNewElements();
+      if (!newElements.isEmpty())
       {
-         errorNodes_.clear();
-         clearErrors_ = false;
+         if (clearErrors_)
+         {
+            errorNodes_.clear();
+            clearErrors_ = false;
+         }
+         errorNodes_.put(error, newElements);
       }
-      errorNodes_.put(error, errorNode);
    }
    
    public void consoleWriteExtendedError(
@@ -271,7 +285,10 @@ public class ShellWidget extends Composite implements ShellDisplay,
    {
       if (errorNodes_.containsKey(error))
       {
-         Node errorNode = errorNodes_.get(error);
+         List<Element> errorNodes = errorNodes_.get(error);
+         if (errorNodes.isEmpty())
+            return;
+         
          clearPendingInput();
          ConsoleError errorWidget = new ConsoleError(
                traceInfo, getErrorClass(), this, command);
@@ -279,9 +296,21 @@ public class ShellWidget extends Composite implements ShellDisplay,
          if (expand)
             errorWidget.setTracebackVisible(true);
          
-         output_.getElement().replaceChild(errorWidget.getElement(), 
-                                           errorNode);
-         
+         boolean replacedFirst = false;
+         for (Element element: errorNodes)
+         {
+            if (!replacedFirst)
+            {
+               // swap widget for first element
+               element.getParentNode().replaceChild(errorWidget.getElement(), element);
+               replacedFirst = true;
+            }
+            else
+            {
+               // and delete the rest of the elements
+               element.removeFromParent();
+            }
+         }
          scrollPanel_.onContentSizeChanged();
          errorNodes_.remove(error);
       }
@@ -296,7 +325,7 @@ public class ShellWidget extends Composite implements ShellDisplay,
    public void consoleWriteOutput(final String output)
    {
       clearPendingInput();
-      output(output, styles_.output(), false);
+      output(output, styles_.output(), false /*isError*/, false /*ignoreLineCount*/);
    }
 
    public void consoleWriteInput(final String input, String console)
@@ -308,7 +337,8 @@ public class ShellWidget extends Composite implements ShellDisplay,
          prompt_.setHTML("");
 
       clearPendingInput();
-      output(input, styles_.command() + KEYWORD_CLASS_NAME, false);
+      output(input, styles_.command() + KEYWORD_CLASS_NAME, false /*isError*/, 
+            false /*ignoreLineCount*/);
    }
    
    private void clearPendingInput()
@@ -319,7 +349,8 @@ public class ShellWidget extends Composite implements ShellDisplay,
 
    public void consoleWritePrompt(final String prompt)
    {
-      output(prompt, styles_.prompt() + KEYWORD_CLASS_NAME, false);
+      output(prompt, styles_.prompt() + KEYWORD_CLASS_NAME, false /*isError*/,
+            false /*ignoreLineCount*/);
       clearErrors_ = true;
    }
 
@@ -339,6 +370,7 @@ public class ShellWidget extends Composite implements ShellDisplay,
       
       input_.setPasswordMode(!showInput);
       clearErrors_ = true;
+      output_.ensureStartingOnNewLine();
    }
 
    public void ensureInputVisible()
@@ -352,95 +384,28 @@ public class ShellWidget extends Composite implements ShellDisplay,
              RStudioGinjector.INSTANCE.getUIPrefs().getThemeErrorClass();
    }
 
+   /**
+    * Send text to the console
+    * @param text Text to output
+    * @param className Text style
+    * @param isError Is this an error message?
+    * @param ignoreLineCount Output without checking buffer length?
+    * @return was this output below the maximum buffer line count?
+    */
    private boolean output(String text,
                           String className,
-                          boolean addToTop)
+                          boolean isError,
+                          boolean ignoreLineCount)
    {
-      if (text.indexOf('\f') >= 0)
-         clearOutput();
+      boolean canContinue = output_.outputToConsole(text, className, 
+                                                    isError, ignoreLineCount);
 
-      Node node;
-      boolean isOutput = StringUtil.isNullOrEmpty(className)
-                         || className.equals(styles_.output());
-
-      if (isOutput && !addToTop && trailingOutput_ != null)
-      {
-         // Short-circuit the case where we're appending output to the
-         // bottom, and there's already some output there. We need to
-         // treat this differently in case the new output uses control
-         // characters to pound over parts of the previous output.
-
-         int oldLineCount = DomUtils.countLines(trailingOutput_, true);
-         trailingOutputConsole_.submit(text);
-         trailingOutput_.setNodeValue(
-               ensureNewLine(trailingOutputConsole_.toString()));
-         int newLineCount = DomUtils.countLines(trailingOutput_, true);
-         lines_ += newLineCount - oldLineCount;
-      }
-      else
-      {
-         Element outEl = output_.getElement();
-
-         text = VirtualConsole.consolify(text);
-         if (isOutput)
-         {
-            VirtualConsole console = new VirtualConsole();
-            console.submit(text);
-            String consoleSnapshot = console.toString();
-
-            // We use ensureNewLine to make sure that even if output
-            // doesn't end with \n, a prompt will appear on its own line.
-            // However, if we call ensureNewLine indiscriminantly (i.e.
-            // on an output that's going to be followed by another output)
-            // we can end up inserting newlines where they don't belong.
-            //
-            // It's safe to add a newline when we're appending output to
-            // the end of the console, because if the next append is also
-            // output, we'll use the contents of VirtualConsole and the
-            // newline we add here will be plowed over.
-            //
-            // If we're prepending output to the top of the console, then
-            // it's safe to add a newline if the next chunk (which is already
-            // there) is something besides output.
-            if (!addToTop ||
-                (!outEl.hasChildNodes()
-                 || outEl.getFirstChild().getNodeType() != Node.TEXT_NODE))
-            {
-               consoleSnapshot = ensureNewLine(consoleSnapshot);
-            }
-
-            node = Document.get().createTextNode(consoleSnapshot);
-            if (!addToTop)
-            {
-               trailingOutput_ = (Text) node;
-               trailingOutputConsole_ = console;
-            }
-         }
-         else
-         {
-            SpanElement span = Document.get().createSpanElement();
-            span.setClassName(className);
-            span.setInnerText(text);
-            node = span;
-            if (!addToTop)
-            {
-               trailingOutput_ = null;
-               trailingOutputConsole_ = null;
-            }
-         }
-
-         if (addToTop)
-            outEl.insertFirst(node);
-         else
-            outEl.appendChild(node);
-
-         lines_ += DomUtils.countLines(node, true);
-      }
-      boolean result = !trimExcess();
-
-      resizeCommand_.nudge();
+      // if we're currently scrolled to the bottom, nudge the timer so that we
+      // will keep up with output
+      if (scrollPanel_.isScrolledToBottom())
+         resizeCommand_.nudge();
       
-      return result;
+      return canContinue;
    }
 
    private String ensureNewLine(String s)
@@ -451,72 +416,101 @@ public class ShellWidget extends Composite implements ShellDisplay,
          return s + '\n';
    }
 
-   private boolean trimExcess()
-   {
-      if (maxLines_ <= 0)
-         return false;  // No limit in effect
-
-      int linesToTrim = lines_ - maxLines_;
-      if (linesToTrim > 0)
-      {
-         lines_ -= DomUtils.trimLines(output_.getElement(),
-                                      lines_ - maxLines_);
-         return true;
-      }
-
-      return false;
-   }
-
    public void playbackActions(final RpcObjectList<ConsoleAction> actions)
    {
+      // Server persists 1000 most recent ConsoleActions in a circular buffer.
+      //
+      // One ConsoleAction can generate multiple lines of output, and we want 
+      // to limit number of lines added to the console's DOM; see trimExcess().
+      //
+      // First walk through the actions in reverse, and determine how many 
+      // lines they will generate (without actually writing anything),
+      // then play-back in normal order. Finally, trim to the max-lines we support
+      // to catch any rounding from final chunk.
+      int lines = 0;
+      int revIndex = actions.length() - 1;
+      for (; revIndex >= 0; revIndex--)
+      {
+         ConsoleAction action = actions.get(revIndex);
+         
+         if (action.getType() == ConsoleAction.INPUT)
+            lines++;
+         
+         lines = lines + StringUtil.newlineCount(action.getData());
+         
+         if (lines > output_.getMaxOutputLines())
+            break;
+      }
+      if (revIndex < 0)
+         revIndex = 0;
+      
+      final int startIndex = revIndex;
+      final int endIndex = actions.length() - 1;
+      
       Scheduler.get().scheduleIncremental(new RepeatingCommand()
       {
-         private int i = actions.length() - 1;
+         private int i = startIndex;
          private int chunksize = 1000;
 
          public boolean execute()
          {
-            int end = i - chunksize;
+            boolean canContinue = false;
+            int end = i + chunksize;
             chunksize = 10;
-            for (; i > end && i >= 0; i--)
+            for (; i <= end && i <= endIndex; i++)
             {
                // User hit Ctrl+L at some point--we're done.
                if (cleared_)
-                  return false;
-
-               boolean canContinue = false;
+               {
+                  canContinue = false;
+                  break;
+               }
 
                ConsoleAction action = actions.get(i);
+               
                switch (action.getType())
                {
                   case ConsoleAction.INPUT:
                      canContinue = output(action.getData() + "\n",
                                           styles_.command() + " " + KEYWORD_CLASS_NAME,
-                                          true);
+                                          false /*isError*/, 
+                                          true /*ignoreLineCount*/);
                      break;
                   case ConsoleAction.OUTPUT:
                      canContinue = output(action.getData(),
                                           styles_.output(),
-                                          true);
+                                          false /*isError*/,
+                                          true /*ignoreLineCount*/);
                      break;
                   case ConsoleAction.ERROR:
                      canContinue = output(action.getData(),
-                                          styles_.error(),
-                                          true);
+                                          getErrorClass(),
+                                          true /*isError*/,
+                                          true /*ignoreLineCount*/);
                      break;
                   case ConsoleAction.PROMPT:
                      canContinue = output(action.getData(),
                                           styles_.prompt() + " " + KEYWORD_CLASS_NAME,
-                                          true);
+                                          false /*isError*/,
+                                          true /*ignoreLineCount*/);
                      break;
                }
                if (!canContinue)
-                  return false;
+               {
+                  break;
+               }                  
             }
-            if (!DomUtils.selectionExists())
-               scrollPanel_.scrollToBottom();
-
-            return i >= 0;
+            
+            if (canContinue)
+            {
+               canContinue = (i <= endIndex);
+            }
+            
+            if (!canContinue)
+            {
+               output_.trimExcess();
+            }
+            return canContinue;
          }
       });
    }
@@ -627,11 +621,8 @@ public class ShellWidget extends Composite implements ShellDisplay,
 
    public void clearOutput()
    {
-      output_.setText("") ;
-      lines_ = 0;
+      output_.clearConsoleOutput();
       cleared_ = true;
-      trailingOutput_ = null;
-      trailingOutputConsole_ = null;
    }
    
    public InputEditorDisplay getInputEditorDisplay()
@@ -681,7 +672,18 @@ public class ShellWidget extends Composite implements ShellDisplay,
       return input_.addKeyPressHandler(handler) ;
    }
    
-   
+   @Override
+   public HandlerRegistration addCapturingKeyUpHandler(KeyUpHandler handler)
+   {
+      return input_.addCapturingKeyUpHandler(handler);
+   }
+
+   @Override
+   public HandlerRegistration addKeyUpHandler(KeyUpHandler handler)
+   {
+      return input_.addKeyUpHandler(handler);
+   }
+
    public int getCharacterWidth()
    {
       return DomUtils.getCharacterWidth(getElement(), styles_.console());
@@ -704,13 +706,12 @@ public class ShellWidget extends Composite implements ShellDisplay,
 
    public int getMaxOutputLines()
    {
-      return maxLines_;
+      return output_.getMaxOutputLines();
    }
    
    public void setMaxOutputLines(int maxLines)
    {
-      maxLines_ = maxLines;
-      trimExcess();
+      output_.setMaxOutputLines(maxLines);
    }
    
    @Override
@@ -731,15 +732,9 @@ public class ShellWidget extends Composite implements ShellDisplay,
       scrollPanel_.onContentSizeChanged();
    }
    
-   private int lines_ = 0;
-   private int maxLines_ = -1;
    private boolean cleared_ = false;
-   private final PreWidget output_ ;
+   private final ConsoleOutputWriter output_;
    private PreWidget pendingInput_ ;
-   // Save a reference to the most recent output text node in case the
-   // next bit of output contains \b or \r control characters
-   private Text trailingOutput_ ;
-   private VirtualConsole trailingOutputConsole_ ;
    private final HTML prompt_ ;
    protected final AceEditor input_ ;
    private final DockPanel inputLine_ ;
@@ -751,7 +746,7 @@ public class ShellWidget extends Composite implements ShellDisplay,
    private final EventBus events_;
    
    // A list of errors that have occurred between console prompts. 
-   private Map<String, Node> errorNodes_ = new TreeMap<String, Node>();
+   private Map<String, List<Element>> errorNodes_ = new TreeMap<String, List<Element>>();
    private boolean clearErrors_ = false;
 
    private static final String KEYWORD_CLASS_NAME = ConsoleResources.KEYWORD_CLASS_NAME;
