@@ -38,7 +38,7 @@ public class RmdTemplateData
             format_extension: "html",
             format_options: [ "toc", "toc_depth", "highlight", "theme", "css", "fig_width", 
                               "fig_height", "fig_caption", "number_sections",
-                              "smart", "self_contained", "keep_md" ],
+                              "smart", "self_contained", "keep_md", "df_print" ],
             format_notes: "Recommended format for authoring (you can switch to PDF or Word output anytime)."
             },
             {
@@ -87,8 +87,8 @@ public class RmdTemplateData
             option_type: "choice", 
             option_default: "default",
             option_nullable: true,
-            option_list: [ "default", "cerulean", "journal", "flatly",
-                           "readable", "spacelab", "united", "cosmo"]
+            option_list: [ "default", "cerulean", "journal", "flatly", "readable", "spacelab", 
+                           "united", "cosmo", "lumen", "paper", "sandstone", "simplex", "yeti" ]
             },
             {
             option_name: "highlight",
@@ -110,6 +110,16 @@ public class RmdTemplateData
                            "espresso", "zenburn", "haddock", "textmate" ]
             },
             {
+            option_name: "df_print",
+            option_ui_name: "Print dataframes as",
+            option_format: "html_document",
+            option_type: "choice", 
+            option_nullable: false,
+            option_default: "paged",
+            option_list: [ "default", "kable", "paged", "tibble" ],
+            option_add_header: true
+            },
+            {
             option_name: "smart",
             option_ui_name: "Use smart punctuation", 
             option_type: "boolean", 
