@@ -24,6 +24,7 @@ import org.rstudio.core.client.command.CommandBinder;
 import org.rstudio.core.client.dom.WindowEx;
 import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.core.client.js.JsObject;
+import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.widget.ModalDialogBase;
 import org.rstudio.core.client.widget.ModalDialogTracker;
 import org.rstudio.core.client.widget.ProgressIndicator;
@@ -601,7 +602,7 @@ public class RSConnect implements SessionInitHandler,
             }));
             HorizontalPanel panel = new HorizontalPanel();
             Image errorImage = 
-                  new Image(MessageDialogImages.INSTANCE.dialog_error());
+                  new Image(new ImageResource2x(MessageDialogImages.INSTANCE.dialog_error2x()));
             errorImage.getElement().getStyle().setMarginTop(1, Unit.EM);
             errorImage.getElement().getStyle().setMarginRight(1, Unit.EM);
             panel.add(errorImage);
@@ -741,7 +742,7 @@ public class RSConnect implements SessionInitHandler,
                result.getSettings().getAsStatic(),
                launchBrowser, 
                RSConnectDeploymentRecord.create(result.getAppName(), 
-                     result.getAppTitle(), result.getAccount(), ""));
+                     result.getAppTitle(), result.getAppId(), result.getAccount(), ""));
 
          // we can't raise the main window if we aren't in desktop mode, so show
          // a dialog to guide the user there
@@ -761,7 +762,7 @@ public class RSConnect implements SessionInitHandler,
                result.getSettings(),
                launchBrowser,
                RSConnectDeploymentRecord.create(result.getAppName(), 
-                     result.getAppTitle(), result.getAccount(), "")));
+                     result.getAppTitle(), result.getAppId(), result.getAccount(), "")));
       }
    }
    
@@ -816,6 +817,7 @@ public class RSConnect implements SessionInitHandler,
                              event.getRecord().getServer(),
                              event.getRecord().getName(), 
                              event.getRecord().getTitle(),
+                             event.getRecord().getAppId(),
                              event.getSettings(),
       new ServerRequestCallback<Boolean>()
       {
