@@ -1,7 +1,7 @@
 /*
  * UIPrefs.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -206,6 +206,11 @@ public class UIPrefs extends UIPrefsAccessor implements UiPrefsChangedHandler
          showSignatureTooltips().setGlobalValue(
                                  newUiPrefs.showSignatureTooltips().getGlobalValue());
          
+         terminalLocalEcho().setGlobalValue(
+                                 newUiPrefs.terminalLocalEcho().getGlobalValue());
+         
+         terminalUseWebsockets().setGlobalValue(
+                                 newUiPrefs.terminalUseWebsockets().getGlobalValue());
          
          /* Diagnostics */
          
@@ -288,10 +293,18 @@ public class UIPrefs extends UIPrefsAccessor implements UiPrefsChangedHandler
          truncateLongLinesInConsoleHistory().setGlobalValue(
                              newUiPrefs.truncateLongLinesInConsoleHistory().getGlobalValue());
          
+         // console handling of ANSI escape codes
+         consoleAnsiMode().setGlobalValue(
+               newUiPrefs.consoleAnsiMode().getGlobalValue());
+         
          // chunk toolbar
          showInlineToolbarForRCodeChunks().setGlobalValue(
                newUiPrefs.showInlineToolbarForRCodeChunks().getGlobalValue());
          
+         // background highlighting
+         highlightCodeChunks().setGlobalValue(
+               newUiPrefs.highlightCodeChunks().getGlobalValue());
+         
          // save all before build
          saveAllBeforeBuild().setGlobalValue(
                              newUiPrefs.saveAllBeforeBuild().getGlobalValue());
@@ -302,6 +315,9 @@ public class UIPrefs extends UIPrefsAccessor implements UiPrefsChangedHandler
       
          // theme
          theme().setGlobalValue(newUiPrefs.theme().getGlobalValue());
+
+         // flat theme
+         getFlatTheme().setGlobalValue(newUiPrefs.getFlatTheme().getGlobalValue());
       
          // default encoding
          defaultEncoding().setGlobalValue(
@@ -455,6 +471,10 @@ public class UIPrefs extends UIPrefsAccessor implements UiPrefsChangedHandler
          // show improved data import dialog
          useDataImport().setGlobalValue(
                newUiPrefs.useDataImport().getGlobalValue());
+         
+         // ignore whitespace in git diff
+         gitDiffIgnoreWhitespace().setGlobalValue(
+               newUiPrefs.gitDiffIgnoreWhitespace().getGlobalValue());
       }
       else if (e.getType().equals(UiPrefsChangedEvent.PROJECT_TYPE))
       {
