@@ -29,6 +29,7 @@ import com.google.gwt.user.client.ui.SplitterResizedEvent;
 import com.google.gwt.user.client.ui.Widget;
 import com.google.inject.Inject;
 import com.google.inject.Provider;
+import com.google.inject.Singleton;
 import com.google.inject.name.Named;
 
 import org.rstudio.core.client.Debug;
@@ -52,6 +53,7 @@ import org.rstudio.core.client.theme.WindowFrame;
 import org.rstudio.core.client.theme.res.ThemeResources;
 import org.rstudio.core.client.widget.ToolbarButton;
 import org.rstudio.studio.client.application.events.EventBus;
+import org.rstudio.studio.client.application.ui.RStudioThemes;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.events.ZoomPaneEvent;
 import org.rstudio.studio.client.workbench.model.ClientState;
@@ -76,6 +78,7 @@ import java.util.List;
  * TODO: Push client state when selected tab or layout changes
  */
 
+@Singleton
 public class PaneManager
 {
    public interface Binder extends CommandBinder<Commands, PaneManager> {}
@@ -219,6 +222,7 @@ public class PaneManager
                       @Named("Source Cpp") final WorkbenchTab sourceCppTab,
                       @Named("R Markdown") final WorkbenchTab renderRmdTab,
                       @Named("Deploy") final WorkbenchTab deployContentTab,
+                      @Named("Terminal") final WorkbenchTab terminalTab,
                       final MarkersOutputTab markersTab,
                       final FindOutputTab findOutputTab,
                       OptionsLoader.Shim optionsLoader)
@@ -246,6 +250,7 @@ public class PaneManager
       renderRmdTab_ = renderRmdTab;
       deployContentTab_ = deployContentTab;
       markersTab_ = markersTab;
+      terminalTab_ = terminalTab;
       optionsLoader_ = optionsLoader;
       
       binder.bind(commands, this);
@@ -253,9 +258,11 @@ public class PaneManager
       PaneConfig config = validateConfig(uiPrefs.paneConfig().getValue());
       initPanes(config);
 
+      int splitterSize = RStudioThemes.isFlat(uiPrefs) ? 7 : 3;
+
       panes_ = createPanes(config);
-      left_ = createSplitWindow(panes_.get(0), panes_.get(1), "left", 0.4);
-      right_ = createSplitWindow(panes_.get(2), panes_.get(3), "right", 0.6);
+      left_ = createSplitWindow(panes_.get(0), panes_.get(1), "left", 0.4, splitterSize);
+      right_ = createSplitWindow(panes_.get(2), panes_.get(3), "right", 0.6, splitterSize);
 
       panel_ = pSplitPanel.get();
       panel_.initialize(left_, right_);
@@ -399,7 +406,12 @@ public class PaneManager
          el = el.getParentElement();
          for (LogicalWindow window : panes_)
          {
-            if (el.equals(window.getActiveWidget().getElement()))
+            Widget activeWidget = window.getActiveWidget();
+            if (activeWidget == null)
+               continue;
+            
+            Element activeEl = activeWidget.getElement();
+            if (el.equals(activeEl))
             {
                targetWindow = window;
                break;
@@ -415,6 +427,51 @@ public class PaneManager
       return getParentLogicalWindow(activeEl);
    }
    
+   @Handler
+   public void onActivateConsolePane()
+   {
+      // Ensure that the console window is activated
+      LogicalWindow consoleWindow = getConsoleLogicalWindow();
+      if (consoleWindow.getState().equals(WindowState.MINIMIZE))
+      {
+         WindowStateChangeEvent event =
+               new WindowStateChangeEvent(WindowState.NORMAL);
+         consoleWindow.onWindowStateChange(event);
+      }
+            
+      // The console tab panel is initialized lazily -- while a console
+      // pane will always be available, the owning tab panel will only
+      // be constructed once a neighbor (e.g. the Terminal) has been
+      // created.
+      if (consoleTabPanel_.isEmpty())
+      {
+         consolePane_.focus();
+      }
+      else
+      {
+         LogicalWindow activeWindow = getActiveLogicalWindow();
+         if (consoleWindow.equals(activeWindow))
+         {
+            consoleTabPanel_.selectNextTab();
+         }
+         else
+         {
+            consoleTabPanel_.selectTab(consoleTabPanel_.getSelectedIndex());
+         }
+      }
+   }
+   
+   @Handler
+   public void onLayoutZoomConsolePane()
+   {
+      if (consoleTabPanel_.isEmpty())
+         consolePane_.focus();
+      else
+         consoleTabPanel_.selectTab(consoleTabPanel_.getSelectedIndex());
+      
+      eventBus_.fireEvent(new ZoomPaneEvent("Console"));
+   }
+   
    @Handler
    public void onLayoutZoomCurrentPane()
    {
@@ -833,6 +890,11 @@ public class PaneManager
       
       toggleWindowZoom(parentWindow, tab);
    }
+   
+   public LogicalWindow getZoomedWindow()
+   {
+      return maximizedWindow_;
+   }
 
    public ConsolePane getConsole()
    {
@@ -857,7 +919,8 @@ public class PaneManager
    private DualWindowLayoutPanel createSplitWindow(LogicalWindow top,
                                                    LogicalWindow bottom,
                                                    String name,
-                                                   double bottomDefaultPct)
+                                                   double bottomDefaultPct,
+                                                   int splitterSize)
    {
       return new DualWindowLayoutPanel(
             eventBus_,
@@ -866,13 +929,14 @@ public class PaneManager
             session_,
             name,
             WindowState.NORMAL,
-            (int) (Window.getClientHeight()*bottomDefaultPct));
+            (int) (Window.getClientHeight()*bottomDefaultPct),
+            splitterSize);
    }
 
    private LogicalWindow createConsole()
    {
       PrimaryWindowFrame frame = new PrimaryWindowFrame("Console", null);
-
+      
       ToolbarButton goToWorkingDirButton =
             commands_.goToWorkingDir().createToolbarButton();
       goToWorkingDirButton.addStyleName(
@@ -881,18 +945,21 @@ public class PaneManager
       LogicalWindow logicalWindow =
             new LogicalWindow(frame, new MinimizedWindowFrame("Console"));
 
-      @SuppressWarnings("unused")
-      ConsoleTabPanel consoleTabPanel = new ConsoleTabPanel(frame,
-                                                            logicalWindow,
-                                                            consolePane_,
-                                                            compilePdfTab_,
-                                                            findOutputTab_,
-                                                            sourceCppTab_,
-                                                            renderRmdTab_,
-                                                            deployContentTab_,
-                                                            markersTab_,
-                                                            eventBus_,
-                                                            goToWorkingDirButton);
+      consoleTabPanel_ = new ConsoleTabPanel(
+            frame,
+            logicalWindow,
+            consolePane_,
+            compilePdfTab_,
+            findOutputTab_,
+            sourceCppTab_,
+            renderRmdTab_,
+            deployContentTab_,
+            markersTab_,
+            terminalTab_,
+            eventBus_,
+            goToWorkingDirButton);
+      
+      consoleTabPanel_.addLayoutStyles(frame.getElement());
       
       return logicalWindow;
    }
@@ -1137,6 +1204,7 @@ public class PaneManager
    private final WorkbenchTab renderRmdTab_;
    private final WorkbenchTab deployContentTab_;
    private final MarkersOutputTab markersTab_;
+   private final WorkbenchTab terminalTab_;
    private final OptionsLoader.Shim optionsLoader_;
    private MainSplitPanel panel_;
    private LogicalWindow sourceLogicalWindow_;
@@ -1150,6 +1218,7 @@ public class PaneManager
    private DualWindowLayoutPanel left_;
    private DualWindowLayoutPanel right_;
    private ArrayList<LogicalWindow> panes_;
+   private ConsoleTabPanel consoleTabPanel_;
    private WorkbenchTabPanel tabSet1TabPanel_;
    private MinimizedModuleTabLayoutPanel tabSet1MinPanel_;
    private WorkbenchTabPanel tabSet2TabPanel_;
