@@ -24,7 +24,9 @@ import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.cellview.ColumnSortInfo;
 import org.rstudio.core.client.cellview.LinkColumn;
 import org.rstudio.core.client.files.FileSystemItem;
+import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.widget.OperationWithInput;
+import org.rstudio.studio.client.ResizableHeader;
 import org.rstudio.studio.client.common.filetypes.FileIconResources;
 import org.rstudio.studio.client.common.filetypes.FileTypeRegistry;
 import org.rstudio.studio.client.workbench.views.files.Files;
@@ -104,7 +106,7 @@ public class FilesList extends Composite
          @Override
          public void onResize(ResizeEvent event)
          {
-            FilesList.this.onResize(event.getWidth(), event.getHeight());
+            FilesList.this.onResize(event.getWidth(), event.getHeight(), false);
          }
       });
    }
@@ -148,7 +150,7 @@ public class FilesList extends Composite
             public ImageResource getValue(FileSystemItem object)
             {
                if (object == parentPath_)
-                  return FileIconResources.INSTANCE.iconUpFolder();
+                  return new ImageResource2x(FileIconResources.INSTANCE.iconUpFolder2x());
                else
                   return fileTypeRegistry.getIconForFile(object);
             }
@@ -222,7 +224,7 @@ public class FilesList extends Composite
          } 
       };  
       sizeColumn.setSortable(true);
-      filesDataGrid_.addColumn(sizeColumn, "Size");
+      filesDataGrid_.addColumn(sizeColumn, new ResizableHeader(filesDataGrid_, "Size"));
       filesDataGrid_.setColumnWidth(sizeColumn, SIZE_COLUMN_WIDTH_PIXELS, Unit.PX);
       
       sortHandler_.setComparator(sizeColumn, new FoldersOnBottomComparator() {
@@ -250,7 +252,7 @@ public class FilesList extends Composite
          } 
       };  
       modColumn.setSortable(true);
-      filesDataGrid_.addColumn(modColumn, "Modified");
+      filesDataGrid_.addColumn(modColumn, new ResizableHeader(filesDataGrid_, "Modified"));
       filesDataGrid_.setColumnWidth(modColumn, MODIFIED_COLUMN_WIDTH_PIXELS, Unit.PX); 
       
       sortHandler_.setComparator(modColumn, new FoldersOnBottomComparator() {
@@ -264,6 +266,11 @@ public class FilesList extends Composite
       return modColumn;
    }
    
+   public void resetColumnWidths()
+   {
+      onResize(true);
+   }
+   
    private void addColumnSortHandler()
    {
       filesDataGrid_.addColumnSortHandler(new Handler() {
@@ -501,7 +508,7 @@ public class FilesList extends Composite
    
    public void redraw()
    {
-      onResize();
+      onResize(false);
       
       // deferred to ensure that browser has responded to our
       // resize request
@@ -515,18 +522,23 @@ public class FilesList extends Composite
       });
    }
    
-   public void onResize()
+   public void onResize(boolean forceResize)
    {
-      onResize(layoutPanel_.getOffsetWidth(), layoutPanel_.getOffsetHeight());
+      onResize(
+            layoutPanel_.getOffsetWidth(),
+            layoutPanel_.getOffsetHeight(),
+            forceResize);
    }
    
-   private void onResize(int width, int height)
+   private void onResize(int width,
+                         int height,
+                         boolean forceResize)
    {
       // Enforce a minimum column width on the name column.
       int newState = width < BOUNDARY_WIDTH_PIXELS ? STATE_SMALL : STATE_LARGE;
 
       // Avoid over-eager updating of column widths.
-      if (state_ == STATE_LARGE && state_ == newState)
+      if (!forceResize && state_ == STATE_LARGE && state_ == newState)
          return;
 
       state_ = newState;
