@@ -1,7 +1,7 @@
 /*
  * CodeBrowserEditingTarget.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -39,6 +39,7 @@ import org.rstudio.core.client.command.KeyboardShortcut;
 import org.rstudio.core.client.events.EnsureHeightHandler;
 import org.rstudio.core.client.events.EnsureVisibleHandler;
 import org.rstudio.core.client.files.FileSystemContext;
+import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.ReadOnlyValue;
@@ -167,6 +168,7 @@ public class CodeBrowserEditingTarget implements EditingTarget
       
       TextEditingTarget.registerPrefs(releaseOnDismiss_, 
                                       prefs_, 
+                                      document.getProjectConfig(),
                                       docDisplay_,
                                       document);
       
@@ -256,7 +258,12 @@ public class CodeBrowserEditingTarget implements EditingTarget
       codeExecution_.executeLastCode();
    }
    
-   
+   @Handler 
+   void onSendToTerminal() 
+   { 
+      codeExecution_.sendSelectionToTerminal(false);
+   } 
+
    @Handler
    void onPrintSourceDoc()
    {
@@ -400,7 +407,7 @@ public class CodeBrowserEditingTarget implements EditingTarget
    @Override
    public ImageResource getIcon()
    {
-      return FileIconResources.INSTANCE.iconSourceViewer();
+      return new ImageResource2x(FileIconResources.INSTANCE.iconSourceViewer2x());
    }
 
    @Override
@@ -409,6 +416,12 @@ public class CodeBrowserEditingTarget implements EditingTarget
       return "R Source Viewer";
    }
    
+   @Override
+   public FileType getFileType()
+   {
+      return null;
+   }
+   
    @Override
    public TextFileType getTextFileType()
    {
@@ -429,6 +442,8 @@ public class CodeBrowserEditingTarget implements EditingTarget
       commands.add(commands_.executeCode());
       commands.add(commands_.executeCodeWithoutFocus());
       commands.add(commands_.executeLastCode());
+      commands.add(commands_.sendToTerminal());
+
       if (SourceWindowManager.isMainSourceWindow())
          commands.add(commands_.popoutDoc());
       else
