@@ -72,8 +72,10 @@ import org.rstudio.studio.client.server.Void;
 import org.rstudio.studio.client.workbench.WorkbenchContext;
 import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UIPrefsAccessor;
 import org.rstudio.studio.client.workbench.views.files.model.FilesServerOperations;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Position;
+import org.rstudio.studio.client.workbench.views.source.editors.text.ui.NewRMarkdownDialog;
 import org.rstudio.studio.client.workbench.views.source.events.FileEditEvent;
 import org.rstudio.studio.client.workbench.views.source.model.DocUpdateSentinel;
 
@@ -193,7 +195,8 @@ public class TextEditingTargetRMarkdownHelper
          });
    }
    
-   public void renderNotebookv2(final DocUpdateSentinel sourceDoc)
+   public void renderNotebookv2(final DocUpdateSentinel sourceDoc,
+         final String viewerType)
    { 
       withRMarkdownPackage("Compiling notebooks from R scripts",
                            false,
@@ -211,7 +214,7 @@ public class TextEditingTargetRMarkdownHelper
                         if (format == null)
                            renderNotebookv2WithDialog(sourceDoc);
                         else
-                           renderNotebookv2(sourceDoc, format);
+                           renderNotebookv2(sourceDoc, format, viewerType);
                      }
                });
             }
@@ -240,7 +243,7 @@ public class TextEditingTargetRMarkdownHelper
          @Override
          public void execute(CompileNotebookv2Options input)
          { 
-            renderNotebookv2(sourceDoc, input.getFormat());
+            renderNotebookv2(sourceDoc, null, input.getFormat());
             
             // save options for this document
             HashMap<String, String> changedProperties 
@@ -265,7 +268,7 @@ public class TextEditingTargetRMarkdownHelper
    }
    
    private void renderNotebookv2(final DocUpdateSentinel sourceDoc,
-                                 String format)
+                                 String format, String viewerType)
    {
       eventBus_.fireEvent(new RenderRmdEvent(sourceDoc.getPath(), 
                                              1, 
@@ -274,7 +277,39 @@ public class TextEditingTargetRMarkdownHelper
                                              null,
                                              false,
                                              RmdOutput.TYPE_STATIC,
-                                             null));
+                                             null,
+                                             getKnitWorkingDir(sourceDoc),
+                                             viewerType));
+   }
+   
+   public String getKnitWorkingDir(DocUpdateSentinel sourceDoc)
+   {
+      // shortcut if we don't support manually specified working directories
+      if (!session_.getSessionInfo().getKnitWorkingDirAvailable())
+         return null;
+      
+      // compute desired working directory type
+      String workingDirType = sourceDoc.getProperty(
+            RenderRmdEvent.WORKING_DIR_PROP,
+            prefs_.knitWorkingDir().getValue());
+
+      String workingDir = null;
+      if (workingDirType == UIPrefsAccessor.KNIT_DIR_PROJECT)
+      {
+         // get the project directory, but if we don't have one (e.g. no
+         // project) use the default working directory for the session
+         FileSystemItem projectDir = 
+               session_.getSessionInfo().getActiveProjectDir();
+         if (projectDir != null)
+            workingDir = projectDir.getPath();
+         if (StringUtil.isNullOrEmpty(workingDir))
+            workingDir = session_.getSessionInfo().getDefaultWorkingDir();
+      }
+      else if (workingDirType == UIPrefsAccessor.KNIT_DIR_CURRENT)
+      {
+         workingDir = workbenchContext_.getCurrentWorkingDir().getPath();
+      }
+      return workingDir;
    }
    
    public void renderRMarkdown(final String sourceFile, 
@@ -284,7 +319,9 @@ public class TextEditingTargetRMarkdownHelper
                                final String paramsFile,
                                final boolean asTempfile,
                                final int type,
-                               final boolean asShiny)
+                               final boolean asShiny,
+                               final String workingDir,
+                               final String viewerType)
    {
       withRMarkdownPackage(type == RmdOutput.TYPE_NOTEBOOK ?
                               "R Notebook" :
@@ -302,7 +339,9 @@ public class TextEditingTargetRMarkdownHelper
                                                    paramsFile,
                                                    asTempfile,
                                                    type,
-                                                   null));
+                                                   null,
+                                                   workingDir,
+                                                   viewerType));
          }
       });
    }
@@ -561,6 +600,19 @@ public class TextEditingTargetRMarkdownHelper
    public void setOutputFormat(String yaml, final String format, 
                                final CommandWithArg<String> onCompleted)
    {
+      // first check to see if the format is already in the YAML; if it is,
+      // we can re-order the formats without roundtripping (this is desirable
+      // because roundtripping goes through the yaml R package which can 
+      // introduce unwanted mutations)
+      YamlTree tree = new YamlTree(yaml);
+      List<String> formats = getOutputFormats(tree);
+      if (formats != null && formats.contains(format))
+      {
+         tree.reorder(Arrays.asList(format));
+         onCompleted.execute(tree.toString());
+         return;
+      }
+
       convertFromYaml(yaml, new CommandWithArg<RmdYamlData>()
       {
          @Override
@@ -963,12 +1015,19 @@ public class TextEditingTargetRMarkdownHelper
       RmdFrontMatterOutputOptions result = RmdFrontMatterOutputOptions.create();
 
       // loop over each option applicable to the new format; if it's
-      // transferable, try to find it in one of the other formats 
+      // transferable, try to find it in one of the other formats. If
+      // it should be added to the header, add it.
       JsArrayString options = template.getFormat(format).getOptions();
       for (int i = 0; i < options.length(); i++)
       {
          String optionName = options.get(i);
          RmdTemplateFormatOption option = template.getOption(optionName);
+
+         if (option.isAddHeader()) {
+            String val = option.getDefaultValue();
+            result.setOptionValue(option, val);
+         }
+
          if (!option.isTransferable())
             continue;
          
@@ -1012,7 +1071,28 @@ public class TextEditingTargetRMarkdownHelper
          outputs.add(tree.getKeyValue(RmdFrontMatter.OUTPUT_KEY));
       return outputs;
    }
-              
+   
+   public void showNewRMarkdownDialog(
+         final OperationWithInput<NewRMarkdownDialog.Result> onComplete)
+   {
+      withRMarkdownPackage(
+         "Creating R Markdown documents",
+         false,
+         new CommandWithArg<RMarkdownContext>()
+      {
+         @Override
+         public void execute(RMarkdownContext context)
+         {
+            new NewRMarkdownDialog(
+               server_,
+               context,
+               workbenchContext_,
+               prefs_.documentAuthor().getGlobalValue(), 
+               onComplete).showModal();
+         }
+      });
+}
+
    private void showKnitrPreviewWarning(WarningBarDisplay display,
                                         String feature, 
                                         String requiredVersion)
